#ifndef _A3CPROJECTBA_INCLUDED_
#define _A3CPROJECTBA_INCLUDED_
#include <Modules/configure.h>
#include <Main/Management/Attributes/BaseAttr.hxx>
#include <Modules/A3CStorageDatabase/inc/repository.hxx>
#include <Modules/User/UserModuleHolder.hxx>
#include <modules\cabin\include\common\pacestringtripple.hxx>
#include <hash_set>
#include <ilviews/base/graphic.h>

class IlvMenuItem;

namespace cabinlayout {
	class PaceCabinItemOAList;
}

namespace Pace{
	namespace A3CProject{

class
A3CProjectBA : public PaceBaseAttr,
			   public virtual Pace::User::UserModuleHolder
{

public:
	A3CProjectBA();
	static A3CProjectBA* getBA();

	virtual const PaceString getID(void);
	virtual int	getAttributeVersion(void);

	virtual void*		userCommand(PaceUserCommandID id,
									void* data,
									const PaceString& basenderID);
	
	virtual	bool loadMe(PaceIstream& in,bool merge);
	virtual	bool saveMe(PaceOstream& out,PaceObjectList* ol = NULL);

	virtual bool isProjectReadOnly() const;
	virtual	bool isProjectCheckedOutByMe(void) const;

	void selectTemplateObjects();
	bool LoadingOfProjectInProgress() const {return _isLoading;}
	static const PaceString ID;
	bool isInTemplate(int objID) const {return _templateIDs.find(objID) != _templateIDs.end();}
	void removeFromTemplate(int objID) {_templateIDs.erase(objID);_removedFromTemplateIDs.insert(objID);}
	void removeSelectionFromTemplate();
	
	/**
	** Set the meta date of the revision iteration based on the layout statistics
	**/
    void setMetaData(boost::shared_ptr<Pace::A3CStorage::PR::RevIt> pRevIt_);

	static void	getIDsFromProject(boost::shared_ptr<Pace::A3CStorage::PR::RevIt> pRevIt_);

	static const std::set<std::string> &getRef_RL_ClassNames();
	
	/**
		Check for cabin items which are part of the template, show a message box if the current action should be
		canceled. If the user wants to proceed, these cabin items are removed from the template.
		If canceled, false is returned, true otherwise.
	*/
	bool checkForTemplateItemsAndRemove(const cabinlayout::PaceCabinItemOAList&);

	void AssignTemplate();

	Pace::A3CStorage::PR::Project const& GetProject() const {return _prjTheProject;}

	void GenerateOutput(std::list<cabinlayout::PaceStringTripple>& AttribLst_);

protected:

	static void		ImportProjectCB(IlvGraphic*, IlvAny a);
	static void		ExportProjectCB(IlvGraphic*, IlvAny a);
	static void		ProjectStatusCB(IlvGraphic*, IlvAny a);
	static void		ProjectReplicationCB(IlvGraphic*, IlvAny a);

	void			loadProject();
	void			loadTheProject(std::string strTemplateExchangeID_ = NULL_UUID);
	void			importProject();
	void			exportProject();
	void			newProject();
	void			projectStatus();
	void			projectReplication();
	bool			saveProject();
	void			saveProjectAs();
	void			replaceTemplate(PaceIstream& proj,PaceIstream& newTemplate);
	
	void			adjustFileMenu(bool bOpened_,bool bReadOnly_);

	/// load objects and attributes from stream and after that store object IDs to objects if objects!=0
	void			loadObjectsAndAttributes(PaceIstream& in,std::set<int>* objects=0,PaceObject* root=0);
	void			clearProject();
	void			afterLoadProject();
	PaceString		getItemLabel(const Pace::A3CStorage::CI::ClassInst& ciItem_,
								 PaceString& strItemName_) const;

	std::string		_showLoadBrowser(const std::string& rstrTitle_ ="");

	std::set<int> _templateIDs;
	std::set<int> _removedFromTemplateIDs;
	static void _addObjIDrec(PaceObject* obj,std::set<int>& s);
	static void		_moveOldObjects(PaceObject* obj,
									PaceObject* parent,
									int parentID,
									const std::map<int,int>& oldtemplatemap,
									const std::set<int>& newtemplate,
									std::vector<std::pair<int,int> >& relocateIDs);
	static void _deleteRuleNode(PaceObject* root);
	static void _deleteObjects(PaceObject* root,std::set<int>& objIDs);
	static void		_findInstallationZoneNodes(PaceObject* obj,
											   std::map<std::string,int>& objectIDs);

	IlvMenuItem* _pAboutItem;
	IlvMenuItem* _pRepliItem;
	bool _isLoading;

	Pace::A3CStorage::PR::Project	_prjTheProject;
	std::string						_strWorkingRevItID;
	PaceString						_strPaceRootLabel;

	static std::set<std::string> s_setRLClassNames;
};

}}

#endif
