#include "Modules/A3CProject/inc/A3CProjectBA.hxx"
#include <plkern/pacelab.hxx>
extern Pacelab* TheApplication;
#include "Main/Management/Undo/UndoManager.hxx"
#include <Main/UserInterface/MainDial/MainDial.hxx>
#include <Main/UserInterface/MainDial/MenuBar.hxx>
#include <main\userinterface\management\grapherobjattr.hxx>
#include <main\userinterface\maindial\grapherattribute.hxx>
#include "Common/UserInterface/WaitingCursorSection.hxx"
#include "Common/Exceptions/LoadingException.hxx"
#include <Modules/Cabin/include/CabinItem/CabinItemOA.hxx>
#include "Modules/Cabin/include/CabinItem/Property/ExplicitViews.hxx"
#include "Modules/Cabin/include/CabinItem/Property/ExplicitViewsWindows.hxx"
#include "Modules/Cabin/include/CabinItem/Property/ComfortClassAssociation.hxx"
#include "Modules/Cabin/include/CabinItem/Property/StorableGraphicOA.hxx"
#include "Modules/Cabin/include/CabinItem/Install/WindowOA.hxx"
#include <Modules/Cabin/include/Management/PUMInterface.hxx>
#include <Modules/Cabin/include/Management/CabinStatisticsOA.hxx>

#include "Modules/Cabin/include/Management/SelectCabinItem.hxx"
#include <Modules\CabinParameter\include\CabinItemTemplateParameter.hxx>
#include "Modules/StowageCompartments/include/FrameSystem/FrameSystemOA.hxx"
#include <Modules/A3CStorage/inc/A3CStorageModule.hxx>
#include <Modules/A3CStorageDatabase/inc/repository.hxx>
#include <Modules/User/UserModule.hxx>
#include <Modules/Public_Project/src/User.hxx>
#include <Modules/Public_Project/src/UserManager/UserRegistrar.hxx>
#include <Modules/Public_Project/src/UserManager/ProjectManagementRegistrar.hxx>
#include <Main/Management/Attributes/ObjectAttrTypeList.hxx>
#include <Modules/Public_Dialogs/FormAlert.hxx>
#include "../inc/Dialogs/NewProjectDialog.hxx"
#include "../inc/Dialogs/OpenProjectDialog.hxx"
#include "../inc/Dialogs/SaveAsProjectDialog.hxx"
#include "../inc/Dialogs/ProjectStatusDialog.hxx"
#include "../inc/Dialogs/ImportProjectDialog.hxx"
#include "../inc/Dialogs/ProjectReplicationDialog.hxx"
#include "../inc/Dialogs/FindProjectsDialog.hxx"
#include "../inc/Dialogs/ItemsChangedByBaselineDialog.hxx"
#include "../inc/Dialogs/dialogutils.hxx"
#include "../inc/A3CProjectModule.hxx"

#include <ilviews/gadgets/fbrowser.h>
#include <ilviews/gadgets/menu.h>
#include <ilviews/gadgets/menubar.h>

using namespace cabinlayout;
using namespace Pace::A3CProject;


// static
std::set<std::string> A3CProjectBA::s_setRLClassNames;


void
A3CProjectBA::ImportProjectCB(IlvGraphic* g, IlvAny a)
{
	IlvAbstractMenu* pMen = (IlvAbstractMenu*)g;
	A3CProjectBA* pBA = (A3CProjectBA*)pMen->getCallbackItem()->getClientData();
	pBA->importProject();
}


void
A3CProjectBA::ExportProjectCB(IlvGraphic* g, IlvAny a)
{
	IlvAbstractMenu* pMen = (IlvAbstractMenu*)g;
	A3CProjectBA* pBA = (A3CProjectBA*)pMen->getCallbackItem()->getClientData();
	pBA->exportProject();
}


void
A3CProjectBA::ProjectStatusCB(IlvGraphic* g, IlvAny a)
{
	IlvAbstractMenu* pMen = (IlvAbstractMenu*)g;
	A3CProjectBA* pBA = (A3CProjectBA*)pMen->getCallbackItem()->getClientData();
	pBA->projectStatus();
}

void
A3CProjectBA::ProjectReplicationCB(IlvGraphic* g, IlvAny a)
{
	IlvAbstractMenu* pMen = (IlvAbstractMenu*)g;
	A3CProjectBA* pBA = (A3CProjectBA*)pMen->getCallbackItem()->getClientData();
	pBA->projectReplication();
}

A3CProjectBA::A3CProjectBA() : 
	PaceBaseAttr(ID,"",ID,Other), _pAboutItem(NULL), _pRepliItem(NULL), _isLoading(false)
{
	setSaveIfNoOAs(true);

	// Init set of Rule-Classnames
	s_setRLClassNames.insert("PaceRuleOA");
	s_setRLClassNames.insert("PaceRuleFolderOA");
	s_setRLClassNames.insert("PaceRulebaseDefault");
	s_setRLClassNames.insert("PaceToplevelFolderOA");
	s_setRLClassNames.insert("PaceFolderSet");

	// delete of dialogs is done by Ilog!

	_strWorkingRevItID = NULL_UUID;

	// add file menu items
	PaceMainDial*   pPacelabMainDial  = NULL;
	PaceMenuBar*    pPacelabMenuBar   = NULL;
	IlvPopupMenu*   pPacelabFileMenu  = NULL;
	
	pPacelabMainDial = TheApplication->getMainDial();
	
	if (pPacelabMainDial != NULL)
		pPacelabMenuBar = pPacelabMainDial->getMenuBar();
	
	if (pPacelabMenuBar != NULL)
	{
		IlvMenuItem * pPacelabFileItem = pPacelabMenuBar->getMainMenuItem(PL_MAINMENU_ITEM_FILE);
		if (pPacelabFileItem)
			pPacelabFileMenu = pPacelabFileItem->getMenu();
	}
	
	if (pPacelabFileMenu)
	{
		IlvMenuItem * pImportMI = pPacelabMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_IMPORT, PL_MAINMENU_ITEM_FILE);
		if (pImportMI)
		{
			pImportMI->setSensitive(IlvTrue);

			IlvPopupMenu * pMenuImp = PaceMenuBar::provideSubMenu(TheApplication->getDisplay(), pImportMI);
			
			IlvMenuItem projectImportMenuItem = IlvMenuItem("&APRJ_'.lab' file...");
			projectImportMenuItem.setCallback(ImportProjectCB);
			projectImportMenuItem.setClientData((IlAny)this);
					
			pMenuImp->addItem(projectImportMenuItem);
		}

		IlvMenuItem * pExportMI = pPacelabMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_EXPORT, PL_MAINMENU_ITEM_FILE);
		
		if (pExportMI)
		{
			pExportMI->setSensitive(IlvTrue);
			IlvPopupMenu * pMenuImp = PaceMenuBar::provideSubMenu(TheApplication->getDisplay(), pExportMI);
			IlvMenuItem projectExportMenuItem = IlvMenuItem("&APRJ_'.lab' file...");
			projectExportMenuItem.setCallback(ExportProjectCB);
			projectExportMenuItem.setClientData((IlAny)this);
					
			pMenuImp->addItem(projectExportMenuItem);		
		}

		_pAboutItem = new IlvMenuItem("&APRJ_Project status...");
		_pAboutItem->setCallback(ProjectStatusCB);
		_pAboutItem->setClientData((IlAny)this);
		_pAboutItem->setSensitive(IlFalse);
		
		IlvUShort nPos = pPacelabFileMenu->getCardinal();
		if (nPos >=2)
			nPos -= 2;	//set nPos to the position before the "File->Quit"
						//and the horizontal divider before the "Quit"
		
		//add a horizontal divider to the menu:
		pPacelabFileMenu->insertItem(new IlvMenuItem(), nPos);
		++nPos;
		//add the new menu-entry for about:
		pPacelabFileMenu->insertItem(_pAboutItem, nPos);

		//add the ProjectReplication menu callback:
		_pRepliItem = new IlvMenuItem("&APRJ_Project replication...");
		_pRepliItem->setCallback(ProjectReplicationCB);
		_pRepliItem->setClientData((IlAny)this);
		pPacelabFileMenu->insertItem(_pRepliItem, nPos);
	}
}

const PaceString A3CProjectBA::getID(void)
{
	return ID;
}

int	A3CProjectBA::getAttributeVersion(void)
{
	return 1001;
}

void*
A3CProjectBA::userCommand(PaceUserCommandID id,
						  void* data,
						  const PaceString& basenderID)
{
	PaceObjectManager* pObjMan = TheApplication->getObjectManager();
	switch(id)
	{
		case PA_UC_LOADPROJECT:
			if(!TheApplication->getMainDial()->checkUser4Destroy())
				return (void*)2;
			loadProject();
			pObjMan->setSaveStatus(true);
			return (void*)-1;

		case PA_UC_NEWPROJECT:
			if(!TheApplication->getMainDial()->checkUser4Destroy())
				return (void*)2;
			newProject();
			return (void*)-1;

		case PA_UC_SAVEPROJECT:
			saveProject();
			return (void*)-1;

		case PA_UC_SAVEPROJECTAS:
			saveProjectAs();
			return (void*)-1;

		case PA_UC_CLOSEPROJECT:
			if(!TheApplication->getMainDial()->checkUser4Destroy())
				return (void*)2;
			clearProject();
			adjustFileMenu(false,true);
			pObjMan->setSaveStatus(true);
			_prjTheProject = Pace::A3CStorage::PR::Project();
			TheApplication->getMainDial()->setWindowTitle();
			return (void*)-1;

		case PA_UC_PROJECTLOADED:
			_pAboutItem->setSensitive(true);
			_pRepliItem->setSensitive(false);
			break;

		case PA_UC_REINIT:
			_pAboutItem->setSensitive(false);
			_pRepliItem->setSensitive(true);
			break;

		case PA_UC_SETMAINWINDOWTITLE:
			{
				static std::string strTitle;
				const Pace::Public_Project::User* pUser = pUserModule->getUser();

				strTitle = IlvGetMessage(TheApplication->getDisplay(),"&APRJ_pacelab A3C: ");
				if (_prjTheProject.getName().empty())
					strTitle += IlvGetMessage(TheApplication->getDisplay(),"&APRJ_-No project-");
				else
				{
					strTitle += _prjTheProject.getName();
					if(isProjectReadOnly())
					{
						strTitle += " ";
						strTitle += IlvGetMessage(TheApplication->getDisplay(), "&APRJ read only");
					}
				}

				adjustFileMenu(pObjMan->getObjectByType("Work Node") ? true : false,
							   !isProjectCheckedOutByMe());
				if (pUser)
				{
					std::string strUserName(pUser->getLogin());
					strUserName = std::string(" (") + strUserName + ")";
					strTitle += strUserName;
				}

				if (!pObjMan->getSaveStatus())
					strTitle += " *";
				return (void *)strTitle.c_str();
			}
			break;
	}
	return 0;
}

bool
A3CProjectBA::loadMe(PaceIstream& in,bool merge)
{
	PaceBaseAttr::loadMe(in,merge);
	int count, value;
	in >> count;
	_templateIDs.clear();
	for(;count>0;--count) {
		in >> value;
		_templateIDs.insert(value);
	}
	if(getLastFileVersion() >= 1001) {
		in >> count;
		_removedFromTemplateIDs.clear();
		for(;count>0;--count) {
			in >> value;
			_removedFromTemplateIDs.insert(value);
		}
	}
	return true;
}


bool
A3CProjectBA::saveMe(PaceOstream& out,PaceObjectList* ol)
{
	PaceBaseAttr::saveMe(out,ol);
	std::set<int>::const_iterator it;
	out << (int)_templateIDs.size() << " ";
	for(it=_templateIDs.begin();it!=_templateIDs.end();++it)
		out << *it << " ";
	out << (int)_removedFromTemplateIDs.size() << " ";
	for(it=_removedFromTemplateIDs.begin();it!=_removedFromTemplateIDs.end();++it)
		out << *it << " ";
	return true;
}


bool
A3CProjectBA::isProjectReadOnly() const
{
	const boost::shared_ptr<A3CStorage::PR::RevIt> pRevIt = _prjTheProject.getRevIt(_strWorkingRevItID);
	return (		!isProjectCheckedOutByMe() 
					||	(_prjTheProject.getRevIt() != pRevIt) 			// latest version?
					||	(pRevIt->GetCurrentLifeCycleStatus() != A3CStorage::PR::INWORK)	);
}


bool
A3CProjectBA::isProjectCheckedOutByMe(void) const
{
	if (!_prjTheProject.getCheckout().empty() &&
		(_prjTheProject.getCheckout() == A3CProjectModule::Instance()->WhoAmI()))
		return true;
	return false;
}


void
A3CProjectBA::_addObjIDrec(PaceObject* obj,std::set<int>& s)
{
	s.insert(obj->getIdentifier());
	const PaceObjectList* children=obj->getChildList();
	for(PaceObjectList::const_iterator it=children->begin();it!=children->end();++it)
		_addObjIDrec(*it,s);	
}


void
A3CProjectBA::loadObjectsAndAttributes(PaceIstream& in,std::set<int>* objects,PaceObject* root)
{
	_isLoading=true;
	PaceObjectManager* om=TheApplication->getObjectManager();
	if(objects)
		objects->clear();
	if(root ? om->mergeObjects(root,in).Ok() : om->loadObjects(in).Ok()) {
		TheApplication->getBaseAttributeManager()->loadAttributes(in);
		if(objects) 
			_addObjIDrec(TheApplication->getObjectManager()->getRoot(),*objects);
	} else {
		Pace::PLKern::Exceptions::LoadingException * pE =
			new Pace::PLKern::Exceptions::LoadingException(
				"void A3CProjectBA::loadObjectsAndAttributes(...)",
				"Error loading objects");
		pE->Throw();
	}
}


std::string
A3CProjectBA::_showLoadBrowser(const std::string& rstrTitle_)
{
	const char* pcTitle = rstrTitle_.empty() ?
				"&open..." : rstrTitle_.c_str();

	IlvFileBrowser fs(TheApplication->getDisplay(),TheApplication->getMainDial()->getSystemView() , ".",
					  0, 0, 0, IlvUseAlwaysSystemDialog);
	static const char* const plabextension[] = {"(*.lab)",
												"(*.*)"};
	static const char* const plabdescription[] = {"LAB - pacelab Project File",
												"All Files"};
	fs.setPatterns(2, plabextension, plabdescription);
	fs.setTitle(IlvGetMessage(TheApplication->getDisplay(), pcTitle));
	PaceString strSelectedFile(fs.get());
	return strSelectedFile;
}


void
A3CProjectBA::adjustFileMenu(bool bOpened_,bool bReadOnly_)
{
	PaceMainDial* pMainDial = TheApplication->getMainDial();
	assert(pMainDial);
	PaceMenuBar* pMenuBar = pMainDial->getMenuBar();
	assert(pMenuBar);

	if (pMenuBar)
	{
		pMenuBar->adjust(bOpened_,bReadOnly_);
	    IlvMenuItem* pMenuItem = pMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_OPEN_PROTO,
													   PL_MAINMENU_ITEM_FILE);
		//  if(pMenuItem != NULL)
		//	pMenuItem->setSensitive(!pProjectModule->StorageSave()); // open default

	    //
		// Adjust Open/New according to action and quiet helper.
		//
		

		bool bCanDoOpen = (pUserModule->CanDo(Pace::Public_Project::UserManager::ProjectmanagementRegistrar::strProjectOpen) == Pace::User::YES);
	    bool bCanDoNew = (pUserModule->CanDo(Pace::Public_Project::UserManager::ProjectmanagementRegistrar::strProjectNewProject) == Pace::User::YES);
	    bool bCanSaveAs = (pUserModule->CanDo(Pace::Public_Project::UserManager::ProjectmanagementRegistrar::strProjectSaveAs) == Pace::User::YES);

	  /*  pMenuItem = pMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_NEW);
        if(pMenuItem != NULL)
			pMenuItem->setSensitive(bCanDoNew);
 
		pMenuItem = pMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_OPEN);
        if(pMenuItem != NULL)
          pMenuItem->setSensitive(bCanDoOpen);
		
	    pMenuItem = pMenuBar->getMenuItem(PL_MAINMENU_ITEM_FILE_SAVEAS);
        if(pMenuItem != NULL)
          pMenuItem->setSensitive(bOpened_ && bCanSaveAs);
	  */
    
    }
}


void
A3CProjectBA::clearProject()
{
	PaceObjectManager* om=TheApplication->getObjectManager();
	PaceBaseAttributeManager* bam=TheApplication->getBaseAttributeManager();
	
	bam->sendUserCmd2BaseAttr(PA_UC_EFFECTIVELYCLOSINGPROJECT, 0, "");
	PaceObject * root = om->getRoot();
	if (root)
	{
		if(_strPaceRootLabel.empty())
		{
			_strPaceRootLabel=root->getName();
		}
		om->killObject(root, false);
	}
	assert (! PaceUndoManager::getInstance()->getInsideUndoAction());
	PaceUndoManager::getInstance()->invalidateUndoStates();
	bam->sendUserCmd2BaseAttr(PA_UC_REINIT,0,PaceString());
}


void
A3CProjectBA::afterLoadProject()
{
	PaceObjectManager* om=TheApplication->getObjectManager();
	PaceBaseAttributeManager* bam=TheApplication->getBaseAttributeManager();
	
	om->loadingComplete();
	PaceObject * pObj = om->getRoot();
	if (pObj)
	{
		pObj->setName(_strPaceRootLabel);
	}
	pObj = om->getObjectByType("Work Node");
	if (pObj)
	{
		pObj->setName(_prjTheProject.getName());
		//om->addAttribute(pObj, PaceGrapherAttribute::ID, new PaceUIGrapherObjAttr("Anchor",	expand));
		PaceUIGrapherObjAttr* pAtt = dynamic_cast<PaceUIGrapherObjAttr*>(om->getAttribute(pObj, PaceGrapherAttribute::ID));
		if(pAtt)
		{
			pAtt->replaceIcon("worknode");
		}

	}
	bam->sendUserCmd2BaseAttr(PA_UC_PROJECTLOADED,(void *)1,PaceString());
	bam->sendUserCmd2BaseAttr(PA_UC_REDRAW,0,PaceString());
	_isLoading=false;
}


void
A3CProjectBA::loadProject()
{
	if (OpenProjectDialog::Instance().expose(_prjTheProject,_strWorkingRevItID))
		loadTheProject();
}


void
A3CProjectBA::loadTheProject(std::string strTemplateExchangeID_)
{
	assert(!_prjTheProject.getRevitVec().empty());

	// check for access rights:
	bool bPermitAccess = true;
	Pace::User::Module *pUserModule = dynamic_cast<Pace::User::Module*>(TheApplication->getModules()->getModuleByID(Pace::User::Module::ModuleID));
	assert(pUserModule);
	const Pace::Public_Project::User *pUser = pUserModule->getUser();

	if ( !pUser )
		bPermitAccess = false;	// no user specified... deny access
	else if (PacePUMInterface::Exemplar().CanDo(A3CProjectModule::strOpenAnyProject, 
																							A3CProjectModule::ID, 
																							A3CProjectModule::strProjectAccessGroup))
		bPermitAccess = true;	// user is Admin... grant access
	else
		bPermitAccess = (_prjTheProject.getAllowedUsers().empty() ||		// access list empty --> grant access to anyone
						 _prjTheProject.IsAllowedUser(pUser->getLogin()));	// test if user in list...
	// grant or revoke access to open project
	if ( !bPermitAccess )
	{
		PaceFormAlert *pFormAlert = TheApplication->getMainDial()->getFormAlert();
        if (pFormAlert)
		pFormAlert->showAlert(PAFA_STOP, "&APRJ_Open Project", "&APRJ_Project_access_denied");
		return;
	}

	boost::shared_ptr<Pace::A3CStorage::PR::RevIt> pProjRevIt = _prjTheProject.getRevIt(_strWorkingRevItID);

	if(strTemplateExchangeID_ == NULL_UUID)
	{
		if (!isProjectReadOnly() &&					// Bugfix #3672: check for edit rights:
			!_prjTheProject.getIsTemplate() &&
			pProjRevIt->getTemplateID() != NULL_UUID)
			strTemplateExchangeID_ = A3CProjectModule::Instance()->check4Update(pProjRevIt);
	}

	Pace::PLKern::UserInterface::WaitingCursorSection ws;

	A3CProjectModule::Instance()->aquireData(*pProjRevIt);
	
	if (strTemplateExchangeID_ != NULL_UUID)
	{
		Pace::A3CStorage::PR::RevIt revItTemplate;
		bool bResult = A3CProjectModule::Instance()->getRevIt(strTemplateExchangeID_,
															  revItTemplate,
															  true);
		assert(bResult);
		if (!bResult)
			return;

		// needed for comparision...

		A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->
									InitRepository(pProjRevIt->getBaselineID(),
									pProjRevIt->getRef_CI_ClassInstID());
		
		// perform reporting of item changes forced by baseline changes...
		// Part 1: remember old settings...

		stdext::hash_set<std::string> hsetInstIDs = pProjRevIt->getRef_CI_ClassInstID();
		stdext::hash_set<std::string>::const_iterator cit = hsetInstIDs.begin();
		std::map<std::string, boost::shared_ptr<A3CStorage::CI::ClassInst> > mapOldClassInsts;
		
		while (cit != hsetInstIDs.end())
		{
			boost::shared_ptr<A3CStorage::CI::ClassInst> clsInst = A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->
																	getClassInst(*cit);
			mapOldClassInsts.insert(std::map<std::string, boost::shared_ptr<A3CStorage::CI::ClassInst> >::value_type
									(clsInst->_InstID, clsInst));
			++cit;
		}

		// exchange template references & init repository with new baseline data...

		pProjRevIt->setTemplate(strTemplateExchangeID_,
								revItTemplate.getBaselineID(),
								revItTemplate.getRuleBaselineID());
		A3CProjectModule::Instance()->aquireData(revItTemplate);
		A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->InitRepository(revItTemplate.getBaselineID(),	pProjRevIt->getRef_CI_ClassInstID());
		A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(revItTemplate.getRuleBaselineID(),	getRef_RL_ClassNames());

		PaceStrStream strStreamProject;
		strStreamProject << pProjRevIt->getData();
		PaceStrStream strStreamTemplate;
		strStreamTemplate << revItTemplate.getData();
		replaceTemplate(strStreamProject, strStreamTemplate);
		strStreamTemplate.freeze(false);
		strStreamProject.freeze(false);

		// perform reporting of item changes forced by baseline changes...
		// Part 2: compare to new baseline...

		hsetInstIDs = pProjRevIt->getRef_CI_ClassInstID();
		cit = hsetInstIDs.begin();
		std::map<std::string, boost::shared_ptr<A3CStorage::CI::ClassInst> >::const_iterator mapCit;
		std::map<PaceString,PaceString> mapStrChangedItems;

		while (cit != hsetInstIDs.end())
		{
			boost::shared_ptr<A3CStorage::CI::ClassInst> clsInst = A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->
																	getClassInst(*cit);
			mapCit = mapOldClassInsts.find(clsInst->_InstID);

			if (mapCit != mapOldClassInsts.end())		// found clsinst in new repository..
			{
				// compare version numbers...
				if ((*mapCit).second->_VersionID != clsInst->_VersionID)
				{
					PaceString strItemName;
					PaceString strItemLabel = getItemLabel(*clsInst, strItemName);
					mapStrChangedItems.insert(std::map<PaceString,PaceString>::value_type(strItemLabel,
																						  strItemName));
				}
			}
			//else { assert(0); } // THINK ABOUT WHICH CASE IS THIS?!?
			++cit;
		}

		// display dial...

		if (mapStrChangedItems.size())
			ItemsChangedByBaselineDialog::Instance().expose(mapStrChangedItems);
	}
	else
	{
		A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->InitRepository(pProjRevIt->getBaselineID(), pProjRevIt->getRef_CI_ClassInstID());
		A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(pProjRevIt->getRuleBaselineID(),	getRef_RL_ClassNames());
		PaceStrStream strStreamProject;
		strStreamProject << pProjRevIt->getData();
		clearProject();
		loadObjectsAndAttributes(strStreamProject,0);
		afterLoadProject();
		strStreamProject.freeze(false);
	}
}


void
A3CProjectBA::newProject()
{
	PaceStrStream strStream;

	if (NewProjectDialog::Instance().expose(_prjTheProject,strStream))
	{
		Pace::PLKern::UserInterface::WaitingCursorSection ws;
		A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->InitRepository(_prjTheProject.getRevIt()->getBaselineID(), _prjTheProject.getRevIt()->getRef_CI_ClassInstID());
		A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(_prjTheProject.getRevIt()->getRuleBaselineID(),	getRef_RL_ClassNames());
		clearProject();
		loadObjectsAndAttributes(strStream,&_templateIDs);
		afterLoadProject();
		_strWorkingRevItID = NULL_UUID;
		_prjTheProject.setCheckout(A3CProjectModule::Instance()->WhoAmI());
		adjustFileMenu(TheApplication->getObjectManager()->getObjectByType("Work Node") ? true : false,
					   !isProjectCheckedOutByMe());
		TheApplication->getMainDial()->setWindowTitle();
	}
}

void
A3CProjectBA::importProject()
{
	PaceStrStream strStream;

	if (ImportProjectDialog::Instance().expose(_prjTheProject,strStream))
	{
		Pace::PLKern::UserInterface::WaitingCursorSection ws;
		A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->InitRepository(_prjTheProject.getRevIt()->getBaselineID());
		A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(_prjTheProject.getRevIt()->getRuleBaselineID(),	getRef_RL_ClassNames());
		clearProject();
		loadObjectsAndAttributes(strStream,0);
		afterLoadProject();
		_strWorkingRevItID = NULL_UUID;		
		_prjTheProject.setCheckout(A3CProjectModule::Instance()->WhoAmI());
		adjustFileMenu(TheApplication->getObjectManager()->getObjectByType("Work Node") ? true : false,
					   !isProjectCheckedOutByMe());
		TheApplication->getMainDial()->setWindowTitle();
	}
}


void
A3CProjectBA::exportProject()
{
	A3CStorageModule::Exemplar(A3CStorageModule::INITIAL)->AllowReferencing(false);
	A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->AllowReferencing(false);
	A3CStorageModule::Exemplar(A3CStorageModule::RULE)->AllowReferencing(false);
	TheApplication->getMainDial()->saveas();
	A3CStorageModule::Exemplar(A3CStorageModule::RULE)->AllowReferencing(true);
	A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->AllowReferencing(true);
	A3CStorageModule::Exemplar(A3CStorageModule::INITIAL)->AllowReferencing(true);
	return;
}


bool
A3CProjectBA::saveProject()
{
	Pace::PLKern::UserInterface::WaitingCursorSection ws;
	boost::shared_ptr<Pace::A3CStorage::PR::RevIt> ri = _prjTheProject.getRevIt(_strWorkingRevItID);

	if(!ri)
	{
		assert(0 && "A3CProjectBA::saveProject(): RevIt not set");
		return false;
	}

	PaceStrStream strStream;
	bool bRet = TheApplication->getMainDial()->
						internalSave(strStream);
	assert(bRet);
	ri->setData(strStream.str());
	getIDsFromProject(ri);
	setMetaData(ri);
	strStream.freeze(false);
	bool result=A3CProjectModule::Instance()->store(_prjTheProject);
	if(result)
	{

		TheApplication->getObjectManager()->setSaveStatus(true);
		adjustFileMenu(true,!isProjectCheckedOutByMe());
		TheApplication->getMainDial()->setWindowTitle();
		TheApplication->getBaseAttributeManager()->sendUserCmd2BaseAttr(PA_UC_REDRAW, 0, ID);
	}
	return result;
}


void
A3CProjectBA::saveProjectAs()
{
	Pace::A3CStorage::PR::Project originalProject(_prjTheProject);
	switch (SaveAsProjectDialog::Instance().expose(_prjTheProject,_strWorkingRevItID))
	{
		case SaveAsProjectDialog::RET_CANCEL:
			return;

		case SaveAsProjectDialog::RET_TEMPLATEOK:
		case SaveAsProjectDialog::RET_OK:
		{
			_strWorkingRevItID = NULL_UUID;
			if(!saveProject())
				_prjTheProject = originalProject;
			return;
		}
	}

}


void
A3CProjectBA::projectStatus()
{

	switch (ProjectStatusDialog::Instance().expose(_prjTheProject,_strWorkingRevItID))
	{
		case ProjectStatusDialog::RET_CANCEL:
			return;

		case ProjectStatusDialog::RET_TEMPLATEUPDATED:
			loadTheProject(ProjectStatusDialog::Instance().getTemplateExchangeID());
			return;
		
		case ProjectStatusDialog::RET_OK:
		{

			return;
		}
	}
}


void
A3CProjectBA::projectReplication()
{
	ProjectReplicationDialog::Instance().expose();
}


void
A3CProjectBA::replaceTemplate(PaceIstream& proj,PaceIstream& newTemplate)
{
	PaceObjectManager* om=TheApplication->getObjectManager();
	Pace::PLKern::UserInterface::WaitingCursorSection ws;
	clearProject();

	// load new template
	std::set<int> newTemplateIDs;
	loadObjectsAndAttributes(newTemplate,&newTemplateIDs);
	std::set<int>::const_iterator it;
	for(it=_removedFromTemplateIDs.begin();it!=_removedFromTemplateIDs.end();++it) {
		PaceObject* obj=om->getObject(*it);
		if(obj) {
			_deleteObjects(obj,newTemplateIDs);
			om->killObject(obj,false);
		}
	}

	// create worknode
	PaceObject* worknode=new PaceObject("worknode",om->getUnmanagedId(),"worknode");
	om->addObject(worknode,om->getRoot());

	// load old project below worknode
	loadObjectsAndAttributes(proj,0,worknode);

	_deleteRuleNode(worknode);
	
	// find objects which don't belong to old template and have a father which does and has a corresponding node
	// in new template
	std::map<int,int> transTblOld;
	for(it=_templateIDs.begin();it!=_templateIDs.end();++it) 
		transTblOld.insert(make_pair(om->translateObjID(*it),*it));

	std::vector<std::pair<int,int> > relocateIDs;
	const PaceObjectList* children=worknode->getChildList();
	for(PaceObjectList::const_iterator lit=children->begin();lit!=children->end();++lit)
		_moveOldObjects(*lit,worknode,-1,transTblOld,newTemplateIDs,relocateIDs);	
	
	// relocate found objects
	for(std::vector<std::pair<int,int> >::const_iterator vit=relocateIDs.begin();vit!=relocateIDs.end();++vit) {
		om->unTieObject(vit->first);
		om->tieObjects(vit->second,vit->first);
	}
	om->killObject(worknode,false);

	// remember new template
	_templateIDs = newTemplateIDs;
	_removedFromTemplateIDs.clear();
	// send rebuildComplete to new template objects, for the other objects this will be done in afterLoadProject()
	for(it=newTemplateIDs.begin();it!=newTemplateIDs.end();++it) {
		PaceObject* obj=om->getObject(*it);
		if(obj) {
			const PaceObjectAttrList* l=obj->getAttribList();
		PaceObjectAttrList::const_iterator it1;
		for(it1= l->begin();it1!=l->end();++it1) 
			(*it1)->rebuildComplete();
		} else {
			assert(0);
		}
	}
	afterLoadProject();
}


void
A3CProjectBA::_deleteRuleNode(PaceObject* root)
{
	if(root->getName() == "&PaceRuleRoot")
		TheApplication->getObjectManager()->killAllChildren(root,false);
	else 
		for(PaceObjectList::const_iterator it=root->getChildList()->begin();it!=root->getChildList()->end();++it)
			_deleteRuleNode(*it);
}

void
A3CProjectBA::_deleteObjects(PaceObject* root,std::set<int>& objIDs)
{
	objIDs.erase(root->getIdentifier());
	for(PaceObjectList::const_iterator it=root->getChildList()->begin();it!=root->getChildList()->end();++it)
		_deleteObjects(*it,objIDs);
}

void A3CProjectBA::_moveOldObjects(PaceObject* obj,
								   PaceObject* parent,int parentID,
								   const std::map<int,int>& oldtemplatemap,
								   const std::set<int>& newtemplate,
								   std::vector<std::pair<int,int> >& relocateIDs)
{
	PaceObjectManager* om=TheApplication->getObjectManager();
	std::map<int,int>::const_iterator it=oldtemplatemap.find(obj->getIdentifier());
	if(it==oldtemplatemap.end() && parentID!=-1) { // this does not belong to old template
		if(newtemplate.find(parentID)!=newtemplate.end()) { // found in new template -> relocate into new template tree
			relocateIDs.push_back(make_pair(obj->getIdentifier(),parentID));
		} 
	} else {
		const PaceObjectList* children=obj->getChildList();
		for(PaceObjectList::const_iterator it1=children->begin();it1!=children->end();++it1)
			_moveOldObjects(*it1,obj,it==oldtemplatemap.end() ? -1 : it->second,oldtemplatemap,newtemplate,relocateIDs);	
	}
}

void A3CProjectBA::selectTemplateObjects()
{
	PaceObjectManager* om=TheApplication->getObjectManager();
	om->sendUserCmd2ObjAttrByTree(om->getRoot(),PA_UC_DESELECT,0,PaceString());
	for(std::set<int>::const_iterator it=_templateIDs.begin();it!=_templateIDs.end();++it) {
		PaceObject* o=om->getObject(*it);
		if(o) {
			PaceObjectAttrList const* l = o->getAttribList();
			for (PaceObjectAttrList::const_iterator it=l->begin();it!=l->end();++it) {
				PaceCabinItemOA* oa=dynamic_cast<PaceCabinItemOA*>(*it);
				if(oa && oa->isVisibleOnScreen()) {
					om->sendUserCmd2ObjAttr(o,PA_UC_SELECT,0,PaceString());
					break;
				}
			}
		}
	}
}

void 
A3CProjectBA::setMetaData(boost::shared_ptr<Pace::A3CStorage::PR::RevIt> pRevIt_)
{
	// reset old values
	int PaxTotal = 0;
	int PaxUDFC = 0;
	int PaxUDBC = 0;
	int PaxUDYC = 0;
	int PaxMDFC = 0;
	int PaxMDBC = 0;
	int PaxMDYC = 0;
	int LavTotal = 0;
	double TrolTotal = 0;

	pRevIt_->setPaxTotal(PaxTotal);
	pRevIt_->setPaxUDFC(PaxUDFC);
	pRevIt_->setPaxUDBC(PaxUDBC);
	pRevIt_->setPaxUDYC(PaxUDYC);
	pRevIt_->setPaxMDFC(PaxMDFC);
	pRevIt_->setPaxMDBC(PaxMDBC);
	pRevIt_->setPaxMDYC(PaxMDYC);
	pRevIt_->setLavTotal(LavTotal);
	pRevIt_->setTrolTotal(TrolTotal);


	// add new values
	PaceCabinStatisticsOA* pStats = 0;
	PaceObjectAttrTypeList<PaceCabinStatisticsOA*> lstStatOA(TheApplication->getObjectManager()->getRoot(),2);
	if(lstStatOA.empty())
		return;

	pStats = lstStatOA.front();
	pStats->CheckClasses();

	// get all compartments
	PaceObjectAttrTypeList<PaceCompartmentOA*> lstCompOA(TheApplication->getObjectManager()->getRoot(), 4);
	PaceObjectAttrTypeList<PaceCompartmentOA*>::const_iterator cit;
	
	for(cit= lstCompOA.begin(); cit != lstCompOA.end(); ++cit)
	{
		// get statistics for compartment
		std::vector<PaceCabinStatisticsOA::classStatistics const*> vecOfClassStats = lstStatOA.front()->getStatistics((*cit)->getObject(),true,true);
		if((*cit)->getObject()->getName() == "&PLCABIN_COMP_BASENAME" )
		{
			for ( unsigned int i = 0; i < vecOfClassStats.size(); ++i )
			{
				PaxTotal += vecOfClassStats[i]->seats;

				if(PaceComfortClassAssociation::getClassType(vecOfClassStats[i]->strClassName) == PaceComfortClassAssociation::CT_FC)
					PaxUDFC += vecOfClassStats[i]->seats;
				else if (PaceComfortClassAssociation::getClassType(vecOfClassStats[i]->strClassName) == PaceComfortClassAssociation::CT_BC)
					PaxUDBC += vecOfClassStats[i]->seats;
				else
					PaxUDYC  += vecOfClassStats[i]->seats;

				LavTotal += vecOfClassStats[i]->lavatories;
				TrolTotal += vecOfClassStats[i]->fullSizeTrollies;
				TrolTotal += (double)(vecOfClassStats[i]->halfSizeTrollies)/2;
			}
		}
		else if ((*cit)->getObject()->getName() == "&PLCABIN_COMP_BASENAME0")
		{
			for ( unsigned int i = 0; i < vecOfClassStats.size(); ++i )
			{
				PaxTotal += vecOfClassStats[i]->seats;

				if(PaceComfortClassAssociation::getClassType(vecOfClassStats[i]->strClassName) == PaceComfortClassAssociation::CT_FC)
					PaxMDFC += vecOfClassStats[i]->seats;
				else if (PaceComfortClassAssociation::getClassType(vecOfClassStats[i]->strClassName) == PaceComfortClassAssociation::CT_BC)
					PaxMDBC += vecOfClassStats[i]->seats;
				else
					PaxMDYC  += vecOfClassStats[i]->seats;

				LavTotal += vecOfClassStats[i]->lavatories;
				TrolTotal += vecOfClassStats[i]->fullSizeTrollies;
				TrolTotal += (double)(vecOfClassStats[i]->halfSizeTrollies)/2;
			}
		}
		else
		{
			for ( unsigned int i = 0; i < vecOfClassStats.size(); ++i )
			{
				PaxTotal += vecOfClassStats[i]->seats;
				LavTotal += vecOfClassStats[i]->lavatories;
				TrolTotal += vecOfClassStats[i]->fullSizeTrollies;
				TrolTotal += (double)(vecOfClassStats[i]->halfSizeTrollies)/2;
			}
		}
	}
	pRevIt_->setPaxTotal(PaxTotal);
	pRevIt_->setPaxUDFC(PaxUDFC);
	pRevIt_->setPaxUDBC(PaxUDBC);
	pRevIt_->setPaxUDYC(PaxUDYC);
	pRevIt_->setPaxMDFC(PaxMDFC);
	pRevIt_->setPaxMDBC(PaxMDBC);
	pRevIt_->setPaxMDYC(PaxMDYC);
	pRevIt_->setLavTotal(LavTotal);
	pRevIt_->setTrolTotal(TrolTotal);
}

void A3CProjectBA::getIDsFromProject(boost::shared_ptr<Pace::A3CStorage::PR::RevIt> pRevIt_)
{
	pRevIt_->clrRef_CI_ClassInstID();
	PaceObjectManager* om=TheApplication->getObjectManager();
	// get IDs from cabin items
	PaceObjectAttrTypeList<PaceCabinItemOA*> cas(om->getRoot());
	for(PaceObjectAttrTypeList<PaceCabinItemOA*>::const_iterator caIt=cas.begin();caIt!=cas.end();++caIt) {
		pRevIt_->insRef_CI_ClassInstID((*caIt)->GetTemplateParameter()->_templateInstID);
		for(int i=1;i<=3;++i) {
			StorableGraphicSmrtPtr sg((*caIt)->GetExplicitViews()->getStorableGraphic(i));
			if(sg.get())
				pRevIt_->insRef_CI_ClassInstID(sg->getParameter()->_templateInstID);
		}
		WindowOA* win=dynamic_cast<WindowOA*>(*caIt);
		if(win) {
			const std::map < int, StorableGraphicSmrtPtr>& graphicMap=win->GetExplicitViews()->getMapGraphics();
			for(std::map < int, StorableGraphicSmrtPtr>::const_iterator it=graphicMap.begin();it!=graphicMap.end();++it) {
				StorableGraphicSmrtPtr sg(it->second);
				if(sg.get())
					pRevIt_->insRef_CI_ClassInstID(sg->getParameter()->_templateInstID);
			}
		}
	}
	// get IDs from storable graphics
	PaceObjectAttrTypeList<StorableGraphicOA*> sgs(om->getRoot());
	for(PaceObjectAttrTypeList<StorableGraphicOA*>::const_iterator sgIt=sgs.begin();sgIt!=sgs.end();++sgIt)
	{
		if ((*sgIt)->GetGraphic() &&
			(*sgIt)->GetGraphic()->getParameter())
			pRevIt_->insRef_CI_ClassInstID((*sgIt)->GetGraphic()->getParameter()->_templateInstID);
		else
			assert(0);
	}

	// get IDs from frame system
	PaceObjectAttrTypeList<Pace::StowageCompartments::FrameSystemOA*> frsys(om->getRoot());
	for(PaceObjectAttrTypeList<Pace::StowageCompartments::FrameSystemOA*>::const_iterator frIt=frsys.begin();frIt!=frsys.end();++frIt) {
		for(std::vector<std::string>::const_iterator it=(*frIt)->getInstanceIDs().begin();it!=(*frIt)->getInstanceIDs().end();++it)
			pRevIt_->insRef_CI_ClassInstID(*it);
	}
}


const std::set<std::string> &
A3CProjectBA::getRef_RL_ClassNames()
{
	return s_setRLClassNames;
}


PaceString
A3CProjectBA::getItemLabel(const Pace::A3CStorage::CI::ClassInst& ciItem_, PaceString& strItemName_) const
{
	// assumption: Cabin Item Name is available under the 'name' identifyer...

	PaceString strReturn = A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->
											GetClassDef(ciItem_._ClassID)->_ClassName;
	strReturn += ": ";


	bool bUnexpected = false;

	std::map<std::string , boost::shared_ptr<Pace::A3CStorage::CI::Param> >::const_iterator cit = ciItem_._param_name.find("name");

	if (cit != ciItem_._param_name.end())
	{
		boost::shared_ptr<Pace::A3CStorage::CI::Param> param = (*cit).second;
		if (param->_DataType != "S")
			bUnexpected = true;
		else
		{
			strItemName_ = param->_ValueStr;
			strReturn += param->_ValueStr;
		}
	}
	else
		bUnexpected = true;

	if (bUnexpected)			// sometimes a goto is nicer, but I dislike the discussions...
	{
		assert(0);				// cabin item without 'name' designator?!?
		strItemName_ = "-?-";
		strReturn += "-?-";
	}

	return strReturn;
}


A3CProjectBA*
A3CProjectBA::getBA()
{
	return dynamic_cast<A3CProjectBA*>(TheApplication->getBaseAttributeManager()->getBaseAttribute(ID));
}

void A3CProjectBA::removeSelectionFromTemplate()
{
    const PaceCabinItemOAList& oal=PaceSelectCabinItem::Exemplar().Selections();
	for(PaceCabinItemOAList::const_iterator it=oal.begin();it!=oal.end();++it) {
		int id=(*it)->getObject()->getIdentifier();
		if(isInTemplate(id))
			removeFromTemplate(id);
	}
}

bool A3CProjectBA::checkForTemplateItemsAndRemove(const cabinlayout::PaceCabinItemOAList& oalist)
{
	bool hasTemplateItem=false;
	for(PaceCabinItemOAList::const_iterator it=oalist.begin();it!=oalist.end();++it) {
		if((*it)->getObject() && isInTemplate((*it)->getObject()->getIdentifier())) {
			hasTemplateItem=true;
			break;
		}
	}
	if(hasTemplateItem) {
		Pace::Dialogs::FormAlert fa(TheApplication->getDisplay());
		PAFA_RETVAL rv = fa.showAlert(PAFA_QUESTION,"&A3CProject",
			"&APRJ_TemplateItemToBeModified",
			NULL,"&^ok","&^cancel",NULL,NULL,NULL);
		if(rv == PAFA_BUT1) {	
			int objId;
			for(PaceCabinItemOAList::const_iterator it=oalist.begin();it!=oalist.end();++it) {
				if((*it)->getObject() && isInTemplate(objId=(*it)->getObject()->getIdentifier()))
					removeFromTemplate(objId);
			}
		} else 
			return false;
	}
	return true;
}

void A3CProjectBA::_findInstallationZoneNodes(PaceObject* obj,std::map<std::string,int>& objectIDs)
{
	if(obj->getParent() && obj->getParent()->getName() == "&CL_CabinLayout") {
		objectIDs.insert(make_pair(
			obj->getName()+"/"+obj->getParent()->getName()+"/"+obj->getParent()->getParent()->getName(),
			obj->getIdentifier()));
	} else {
		for(PaceObjectList::const_iterator it=obj->getChildList()->begin();it!=obj->getChildList()->end();++it) 
			_findInstallationZoneNodes(*it,objectIDs);	
	}
}

void A3CProjectBA::AssignTemplate()
{
	Pace::PLKern::UserInterface::WaitingCursorSection ws;
	clearProject();

	// set template in project, adjust baselines
	boost::shared_ptr<Pace::A3CStorage::PR::RevIt> projectRevIt(_prjTheProject.getUnconstRevIt(_strWorkingRevItID));
	boost::shared_ptr<Pace::A3CStorage::PR::Aircraft> aircraft;
	A3CProjectModule::Instance()->getAircraft(projectRevIt->getAircraftID(),aircraft);
	Pace::A3CStorage::PR::RevIt	templateRevIt;
	A3CProjectModule::Instance()->getRevIt(aircraft->getTemplateID(), templateRevIt);		
	A3CProjectModule::Instance()->aquireData(templateRevIt);
	A3CStorageModule::Exemplar(A3CStorageModule::COMMON)->InitRepository(templateRevIt.getBaselineID(),	projectRevIt->getRef_CI_ClassInstID());
	A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(templateRevIt.getRuleBaselineID(),	getRef_RL_ClassNames());
	projectRevIt->setTemplate(aircraft->getTemplateID(),
							  templateRevIt.getBaselineID(),
							  templateRevIt.getRuleBaselineID());

	// load project and template data
	PaceStrStream strStreamProject;
	strStreamProject << projectRevIt->getData();
	PaceStrStream strStreamTemplate;
	strStreamTemplate << templateRevIt.getData();

	PaceObjectManager* om=TheApplication->getObjectManager();

	// load new template
	std::set<int> newTemplateIDs;
	loadObjectsAndAttributes(strStreamTemplate,&newTemplateIDs);
	std::set<int>::const_iterator it;
	for(it=_removedFromTemplateIDs.begin();it!=_removedFromTemplateIDs.end();++it) {
		PaceObject* obj=om->getObject(*it);
		if(obj) {
			_deleteObjects(obj,newTemplateIDs);
			om->killObject(obj,false);
		}
	}
	std::map<std::string,int> templateInstallSides;
	_findInstallationZoneNodes(om->getRoot(),templateInstallSides);

	// create worknode
	PaceObject* worknode=new PaceObject("worknode",om->getUnmanagedId(),"worknode");
	om->addObject(worknode,om->getRoot());

	// load old project below worknode
	loadObjectsAndAttributes(strStreamProject,0,worknode);

	strStreamTemplate.freeze(false);
	strStreamProject.freeze(false);

	_deleteRuleNode(worknode);

	std::map<std::string,int> projectInstallSides;
	_findInstallationZoneNodes(worknode,projectInstallSides);
	
	// move objects from project to template
	for(std::map<std::string,int>::const_iterator cit=projectInstallSides.begin();cit!=projectInstallSides.end();++cit) {
		std::map<std::string,int>::const_iterator itTmpl=templateInstallSides.find(cit->first);
		if(itTmpl != templateInstallSides.end()) {
			PaceObject* obj=om->getObject(cit->second);
			assert(obj);
			PaceObject* objTmpl=om->getObject(itTmpl->second);
			assert(objTmpl);
			while(!obj->getChildList()->empty()) {
				PaceObject* objToMove=obj->getChildList()->front();
				om->unTieObject(objToMove);
				om->tieObjects(objTmpl,objToMove);
			}
		}
	}

	om->killObject(worknode,false);

	// remember new template
	_templateIDs = newTemplateIDs;
	_removedFromTemplateIDs.clear();
	// send rebuildComplete to new template objects, for the other objects this will be done in afterLoadProject()
	for(it=newTemplateIDs.begin();it!=newTemplateIDs.end();++it) {
		PaceObject* obj=om->getObject(*it);
		if(obj) {
			const PaceObjectAttrList* l=obj->getAttribList();
		PaceObjectAttrList::const_iterator it1;
		for(it1= l->begin();it1!=l->end();++it1) 
			(*it1)->rebuildComplete();
		} else {
			assert(0);
		}
	}
	afterLoadProject();
}

const PaceString A3CProjectBA::ID("A3CProjectBA");

void A3CProjectBA::GenerateOutput(std::list<cabinlayout::PaceStringTripple>& AttribLst_)
{
	AttribLst_.push_back(PaceStringTripple("A3C_AircraftFamily", ListableCriterionPool::GetInstance().LCACFamilyExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_AircraftType", ListableCriterionPool::GetInstance().LCACTypeExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_AircraftSeries", ListableCriterionPool::GetInstance().LCACSeriesExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_Revision", ListableCriterionPool::GetInstance().LCRevisionExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_Iteration", ListableCriterionPool::GetInstance().LCIterationExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_IataCode", _prjTheProject.getAirlineICAO()));
	AttribLst_.push_back(PaceStringTripple("A3C_AirlineName", _prjTheProject.getAirlineName()));
	AttribLst_.push_back(PaceStringTripple("A3C_TotalPaxNum", ListableCriterionPool::GetInstance().LCPAXTotalExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_StdSpec", ListableCriterionPool::GetInstance().LCStdSpecExtractionFunc(_prjTheProject, _strWorkingRevItID)));
	AttribLst_.push_back(PaceStringTripple("A3C_CCG", ListableCriterionPool::GetInstance().LCCCGIssueNameExtractionFunc(_prjTheProject, _strWorkingRevItID)));
}