#include "../inc/A3CStorage.h"
#include "../inc/A3CAdminTool.hxx"
#include "../inc/A3CStorageModule.hxx"
#include "Modules\A3CProject\inc\A3CProjectModule.hxx"

#include <plkern/pacelab.hxx>
#include <Modules/Cabin/include/CabinItem/CabinItemBA.hxx>
#include <modules/cabin/include/Management/cabinlayoutglobal.hxx>
#include <Modules/Cabin/include/Management/StorageAccessMgr.hxx>
#include <Modules/Cabin/include/Management/CabinItemStorageAccessor.hxx>
#include <Modules/Cabin/include/Management/PUMInterface.hxx>
#include <Main/UserInterface/MainDial/MainDial.hxx>
#include <Modules/Public_Storage/src/Storage/Storable.hxx>

#include <Main/UserInterface/Vaults/ObjTemplVault.hxx>
#include <Main/UserInterface/Management/ObjectTemplate.hxx>

#include <ilviews/gadgets/scombo.h>
#include <ilviews/gadgets/amenu.h>
#include <ilviews/gadgets/toolbar.h>
#include <ilviews/gadgets/tree.h>
#include <ilviews/gadgets/matrix.h>
#include <ilviews/gadgets/button.h>
#include <ilviews/graphics/tooltip.h>

#include <sstream>

#ifdef PACE_DEBUG_NEW
#define new PACE_DEBUG_NEW
#endif

using namespace Pace;
using namespace A3CStorage;
using namespace A3CProject;

extern Pacelab* TheApplication;

static bool ProjectLoaded()
{
	return TheApplication->getObjectManager()->getObjectByType("Work Node") ? true:false;
}

class ATNewCIBaselineDlg : public cabinlayout::PaceGadgetContainer
{
public:
	static void ILVCALLBACK DefineBaselineCb(IlvGraphic*, IlvAny pDlg_)		
	{ATNewCIBaselineDlg* pDlg=NULL;
																		
		pDlg = reinterpret_cast<ATNewCIBaselineDlg*>(pDlg_);
		if(pDlg) {
			IlvTextField* pT=(IlvTextField*)pDlg->getObject("EDT_NewVersion");	
			Pace::A3CStorageModule::Exemplar()->DefineBaseline(pT->getLabel());
			pDlg->hide();

			A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
			A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();
		}
	}

	static void ILVCALLBACK CloseCb(IlvGraphic*, IlvAny pDlg_)		
	{ATNewCIBaselineDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATNewCIBaselineDlg*>(pDlg_);
		if(pDlg) {
			pDlg->hide();		
		}
	}

	static void ILVCALLBACK windowCloseCB(IlvView* pView_, IlvAny)
	{
		pView_->hide();
	}

	ATNewCIBaselineDlg() :
		PaceGadgetContainer(TheApplication->getDisplay(),
		"ATNewCIBaselineDlg",
		"Define New Baseline",
		IlvRect(0, 0, 200, 150),
        IlvNoResizeBorder,
        IlvFalse,
        IlvFalse,
        TheApplication->getMainView())
	{IlvTextField* pTf;

		readFile("Resource/Modules/A3CStorage/Ilv/NewBaselineDlg.ilv");		
		((IlvButton*)getObject("BTN_Ok"))->setCallback(DefineBaselineCb, (IlvAny)this);
		((IlvButton*)getObject("BTN_CloseToo"))->setCallback(CloseCb, (IlvAny)this);
		setDestroyCallback(windowCloseCB, this);
		pTf=(IlvTextField*)getObject("EDT_NewVersion");
		assert(pTf);
		if(pTf)
			pTf->setLabel(IlvGetMessage(TheApplication->getDisplay(),"&<new baseline>"));
	}		
};



class ATNewRLBaselineDlg : public cabinlayout::PaceGadgetContainer
{
public:
	static void ILVCALLBACK DefineBaselineCb(IlvGraphic*, IlvAny pDlg_)		
	{ATNewRLBaselineDlg* pDlg=NULL;
																		
		pDlg = reinterpret_cast<ATNewRLBaselineDlg*>(pDlg_);
		if(pDlg) {
			IlvTextField* pT=(IlvTextField*)pDlg->getObject("EDT_NewVersion");	
			A3CStorageModule::Exemplar(A3CStorageModule::RULE)->DefineBaseline(pT->getLabel());
			pDlg->hide();

			A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();
			A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
		}
	}

	static void ILVCALLBACK CloseCb(IlvGraphic*, IlvAny pDlg_)		
	{ATNewRLBaselineDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATNewRLBaselineDlg*>(pDlg_);
		if(pDlg) {
			pDlg->hide();		
		}
	}

	static void ILVCALLBACK windowCloseCB(IlvView* pView_, IlvAny)
	{
		pView_->hide();
	}

	ATNewRLBaselineDlg() :
		PaceGadgetContainer(TheApplication->getDisplay(),
		"ATNewRLBaselineDlg", 
		"Define New Baseline",
		IlvRect(0, 0, 200, 150),
        IlvNoResizeBorder,
        IlvFalse,
        IlvFalse,
        TheApplication->getMainView())
	{IlvTextField* pTf;

		readFile("Resource/Modules/A3CStorage/Ilv/NewBaselineDlg.ilv");		
		((IlvButton*)getObject("BTN_Ok"))->setCallback(DefineBaselineCb, (IlvAny)this);
		((IlvButton*)getObject("BTN_CloseToo"))->setCallback(CloseCb, (IlvAny)this);
		setDestroyCallback(windowCloseCB, this);
		pTf=(IlvTextField*)getObject("EDT_NewVersion");
		assert(pTf);
		if(pTf)
			pTf->setLabel(IlvGetMessage(TheApplication->getDisplay(),"&<new baseline>"));
	}		
};



class ATSelectTemplateDlg : public cabinlayout::PaceGadgetContainer
{
public:

	static void ILVCALLBACK SelectCb(IlvGraphic*, IlvAny pDlg_)		
	{ATSelectTemplateDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATSelectTemplateDlg*>(pDlg_);
		if(pDlg) {
			pDlg->OnSelect();		
		}
	}

	static void ILVCALLBACK OkCb(IlvGraphic*, IlvAny pDlg_)		
	{ATSelectTemplateDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATSelectTemplateDlg*>(pDlg_);
		if(pDlg) {
			pDlg->OnOk();		
		}
	}

	static void ILVCALLBACK CancelCb(IlvGraphic*, IlvAny pDlg_)		
	{ATSelectTemplateDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATSelectTemplateDlg*>(pDlg_);
		if(pDlg) {
			pDlg->OnCancel();		
		}
	}

	static void ILVCALLBACK windowCloseCB(IlvView* pView_, IlvAny pDlg_)
	{ATSelectTemplateDlg* pDlg=NULL;
													
		pDlg = reinterpret_cast<ATSelectTemplateDlg*>(pDlg_);
		if(pDlg) {
			pDlg->OnCancel();		
		}
	}

    ATSelectTemplateDlg(bool& rValid_, std::string& rTemplateName_, std::string& rTemplateID_, const std::list<boost::shared_ptr<PR::Project> >& lstProj_)
        :   PaceGadgetContainer(TheApplication->getDisplay(),
		"ATSelectTemplateDlg",
		"Define New Baseline",
		IlvRect(0, 0, 200, 150),
        IlvNoResizeBorder,
        IlvFalse,
        IlvFalse,
        TheApplication->getMainView()),
		_rTemplateName(rTemplateName_),
		_rTemplateID(rTemplateID_),
		_rValid(rValid_)
    {
        std::list<boost::shared_ptr<PR::Project> >::const_iterator citP;

		_rValid=false;
		readFile("Resource/Modules/A3CStorage/Ilv/SelectTemplateDlg.ilv");		
		_pBtnOk=(IlvButton*)getObject("BTN_STOk");
		assert(_pBtnOk);
		_pBtnOk->setCallback(OkCb, this);
		_pBtnOk->setSensitive(IlFalse);
		_pBtnCncl=(IlvButton*)getObject("BTN_STCancel");
		assert(_pBtnCncl);
		_pBtnCncl->setCallback(CancelCb, this);
		setDestroyCallback(windowCloseCB, this);
		_pTree=(IlvTreeGadget*)getObject("GAD_STTree");
		assert(_pTree);
		_pTree->removeAllItems();
		_pTree->setCallback(_pTree->SelectCallbackType(), SelectCb, this);
		
		IlvTreeGadgetItem *pItmP, *pItmR;

		pItmP=_pTree->addItem(0, "no template");
		pItmP->setClientData((IlAny)0);
		_idmap[0]=NULL_UUID;
		int cd=1;
        for(citP=lstProj_.begin(); citP!=lstProj_.end(); citP++)
		{
			if((*citP)->getIsTemplate())
            {std::vector<boost::shared_ptr<PR::RevIt> >::const_iterator citR;
			 
				pItmP=_pTree->addItem(0, (*citP)->getName().c_str());
				pItmP->setClientData((IlAny)-1);
				for(citR=(*citP)->getRevitVec().begin(); citR!=(*citP)->getRevitVec().end(); citR++)
                {
                    std::stringstream ss;
					
					ss << (*citR)->getRevision() << "/" << (*citR)->getIteration();
                    pItmR = _pTree->addItem(pItmP, ss.str().c_str());
                    _idmap[cd] = (*citR)->getRevItID();
					pItmR->setClientData((IlAny)cd);
					cd++;
				}	
			}
		}
		moveToScreen();
		showModal();
	}
protected:
	void OnSelect()
    {
        IlvTreeGadgetItem* pItm;
	 IlvTreeGadgetItem* pPItm;

		pItm=_pTree->getFirstSelectedItem();
		if(pItm)
		{
			int cd = (int)pItm->getClientData();
			if(_idmap.find(cd)!=_idmap.end())
			{
				_rTemplateID=_idmap[cd];
				_pBtnOk->setSensitive(IlTrue);
				if(_rTemplateID!=NULL_UUID)
				{
					pPItm=pItm->getParent();
					_rTemplateName=pPItm->getLabel();
					_rTemplateName+=" ";
					_rTemplateName+=pItm->getLabel();
				}
				else
				{
					_rTemplateName="<not set>";
				}
			}
			else
			{
				_pBtnOk->setSensitive(IlFalse);
			}
			_pBtnOk->reDraw();
		}
	}

	void OnOk()
	{
		_rValid=true;
		hide();
	}
	void OnCancel()
	{
		_rValid=false;
		hide();
	}
	std::string& _rTemplateName; 
	std::string& _rTemplateID;
	bool& _rValid;
	IlvButton* _pBtnOk;
	IlvButton* _pBtnCncl;
	IlvTreeGadget* _pTree;
	std::map<int, std::string> _idmap;
};



const PaceString A3CAdminToolDlg::_strDlgName("AdminToolDlg");
const PaceString A3CAdminToolDlg::_strIlvFilename("Resource/Modules/A3CStorage/Ilv/AdminToolDlg.ilv");

A3CAdminToolDlg* A3CAdminToolDlg::_exemplar=NULL;

static void ILVCALLBACK _windowCloseCB(IlvView* , IlvAny)
{
	A3CAdminToolDlg::Exemplar()->hide();
}

A3CAdminToolDlg::A3CAdminToolDlg()
  : PaceGadgetContainer(TheApplication->getDisplay(),
						_strDlgName.c_str(),
						IlvGetMessage(TheApplication->getDisplay(), cabinlayout::PaceCabinLayoutGlobal::Exemplar().Wintitle),
						IlvRect(0, 0, 320, 344),
						IlvNoResizeBorder,
						IlvFalse,
						IlvFalse,
						TheApplication->getMainView()),
						_pItmBLDefine(NULL),
						_pItmBLCommit(NULL),
						_pItmBLRollback(NULL),
						_pCITree(NULL),
						_BLShowDiff(false),
						_pItmRuleBLDefine(NULL),
						_pItmRuleBLCommit(NULL),
						_pItmRuleBLRollback(NULL),
						_pRLTree(NULL),
						_RuleBLShowDiff(false),
						_editMode(false),
						_pItmPREnterEditCust(NULL),
						_pItmPRUploadCust(NULL),
						_pItmPREnterEditOrg(NULL),
						_pItmPRUploadOrg(NULL),
						_pOrgTree(NULL),
						_pItmOrgAddSib(NULL),
						_pItmOrgAddChi(NULL),
						_pItmOrgDel(NULL),
						_pCcgList(NULL),
						_pItmCcgNew(NULL),
						_pItmCcgDel(NULL),
						_pCustMtx(NULL),
						_pItmCustNew(NULL),
						_pItmCustDel(NULL),
						_custChanged(false),
						_pAcMtx(NULL),
						_pItmAcNew(NULL),
						_pItmAcDel(NULL)
{
	PaceGadgetContainer::readFile(_strIlvFilename.c_str());
	setDestroyCallback(_windowCloseCB, this);

	((IlvButton*)getObject("BTN_Close"))->setCallback(OnClose);

	// user data
	registerCallback("UploadUserCb", OnUploadUserData);
	_pBtnUploadUser = (IlvButton*)getObject("BTN_UploadUser");

	// cabin item baseline
	registerCallback("DefineBaselineCb", OnDefineBaseline);
	registerCallback("CommitBaselineCb", OnCommitBaseline);
	registerCallback("RollbackBaselineCb", OnRollbackBaseline);
	registerCallback("ShowDiffCb", OnShowDiffOnly);
	registerCallback("ExpandItemCb", OnExpandItem);
	registerCallback("ShowItemCb", OnShowItem);
	((IlvScrolledComboBox*)getObject("CBX_Version"))->setCallback(OnVersionSelect);

	IlvToolBar* toolbar = (IlvToolBar*)getObject("TLB_Baseline");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmDefine")
				{
					_pItmBLDefine=items[i];
				}
				else if(name=="ItmCommit")
				{
					_pItmBLCommit=items[i];
				}
				else if(name=="ItmRollback")
				{
					_pItmBLRollback=items[i];
				}
			}
		}
	}
	assert(_pItmBLDefine && _pItmBLCommit && _pItmBLRollback);
	_pCITree=(IlvTreeGadget*)getObject("Tree");
	assert(_pCITree);

	// rule baseline
	registerCallback("RuleDefineBaselineCb", OnRuleDefineBaseline);
	registerCallback("RuleCommitBaselineCb", OnRuleCommitBaseline);
	registerCallback("RuleRollbackBaselineCb", OnRuleRollbackBaseline);
	registerCallback("RuleShowDiffCb", OnRuleShowDiffOnly);
	registerCallback("RuleExpandItemCb", OnRuleExpandItem);
	registerCallback("RuleShowItemCb", OnRuleShowItem);
	((IlvScrolledComboBox*)getObject("CBX_RuleVersion"))->setCallback(OnRuleVersionSelect);

	toolbar = (IlvToolBar*)getObject("TLB_RuleBaseline");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmDefine")
				{
					_pItmRuleBLDefine=items[i];
				}
				else if(name=="ItmCommit")
				{
					_pItmRuleBLCommit=items[i];
				}
				else if(name=="ItmRollback")
				{
					_pItmRuleBLRollback=items[i];
				}
			}
		}
	}
	assert(_pItmRuleBLDefine && _pItmRuleBLCommit && _pItmRuleBLRollback);
	_pRLTree=(IlvTreeGadget*)getObject("RuleTree");
	assert(_pRLTree);

	// PR Admin
	registerCallback("PREnterEditCb", OnPREnterEdit);
	registerCallback("PRUploadCb", OnPRUpload);
	
	toolbar = (IlvToolBar*)getObject("TLB_OrgAdmin");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmEnter")
				{
					_pItmPREnterEditOrg=items[i];
				}
				else if(name=="ItmUpload")
				{
					_pItmPRUploadOrg=items[i];
				}
			}
		}
	}
	assert(_pItmPREnterEditOrg && _pItmPRUploadOrg );
	// organisation
	toolbar = (IlvToolBar*)getObject("TLB_Org");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmOrgAddSibling")
				{
					_pItmOrgAddSib=items[i];
				}
				else if(name=="ItmOrgAddChild")
				{
					_pItmOrgAddChi=items[i];
				}
				else if(name=="ItmOrgDeleteEntry")
				{
					_pItmOrgDel=items[i];
				}
			}
		}
	}
	assert(_pItmOrgAddSib && _pItmOrgAddChi && _pItmOrgDel);
	registerCallback("OrgSelectCb", OnOrgSelect);
	registerCallback("OrgAddSiblingCb", OnOrgAddSibling);
	registerCallback("OrgAddChildCb", OnOrgAddChild);
	registerCallback("OrgDeleteEntryCb", OnOrgDeleteEntry);
	registerCallback("OrgRenameCb", OnOrgRename);
	_pOrgTree=(IlvTreeGadget*)getObject("GAD_Org");
	assert(_pOrgTree);


	// ccg
	toolbar = (IlvToolBar*)getObject("TLB_CCG");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmNew")
				{
					_pItmCcgNew=items[i];
				}
                else if(name=="ItmDel")
				{
					_pItmCcgDel=items[i];
				}
			}
		}
	}
    
	assert(_pItmCcgNew && _pItmCcgDel);
	registerCallback("CcgSelectCb", OnCcgSelect);
	registerCallback("CcgNewCb", OnCcgNew);
	registerCallback("CcgRenameCb", OnCcgRename);
	registerCallback("CcgDeleteCb", OnCcgDelete);
	_pCcgList=(IlvStringList*)getObject("GAD_CCG");
	assert(_pCcgList);

	// AC
	toolbar = (IlvToolBar*)getObject("TLB_AC");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmNew")
				{
					_pItmAcNew=items[i];
				}
                else if(name=="ItmDel")
				{
					_pItmAcDel=items[i];
				}
			}
		}
	}
    
	assert(_pItmAcNew && _pItmAcDel);
	registerCallback("AcNewCb", OnAcNew);
	registerCallback("AcDeleteCb", OnAcDelete);
	_pAcMtx=(IlvMatrix*)getObject("GAD_AC");
	assert(_pAcMtx);
	
	

	// customers
	toolbar = (IlvToolBar*)getObject("TLB_CustAdmin");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmEnter")
				{
					_pItmPREnterEditCust=items[i];
				}
                else if(name=="ItmUpload")
				{
					_pItmPRUploadCust=items[i];
				}
			}
		}
	}
	assert(_pItmPREnterEditCust && _pItmPRUploadCust);
	
	toolbar = (IlvToolBar*)getObject("TLB_Cust");
	assert(toolbar);
	if(toolbar) 
	{
		IlUShort count;
		IlvMenuItem* const* items = toolbar->getItems(count);
		for(int i=0; i<count; ++i)
		{
			IlvMenuItemType type = items[i]->getType();
			if(type != IlvSeparatorItem)
			{
				string name = items[i]->getName();
				if(name=="ItmNew")
				{
					_pItmCustNew=items[i];
				}
                else if(name=="ItmDel")
				{
					_pItmCustDel=items[i];
				}
			}
		}
	}
    
	assert(_pItmCustNew && _pItmCustDel);
	registerCallback("CustomerNewCb", OnCustomerNew);
	registerCallback("CustomerDeleteCb", OnCustomerDelete);
	_pCustMtx=(IlvMatrix*)getObject("GAD_Cust");
	assert(_pCustMtx);

	assert(A3CProjectModule::Instance());
}

A3CAdminToolDlg::~A3CAdminToolDlg()
{

}

/* static*/ 
A3CAdminToolDlg* A3CAdminToolDlg::Exemplar( )
{
	if (!_exemplar)
	{
		_exemplar = new A3CAdminToolDlg();
	}

	return _exemplar;
}

void A3CAdminToolDlg::Show()
{
	if(cabinlayout::PacePUMInterface::Exemplar().CanDo(
              A3CStorageModule::strOpenAdminTool, 
			  A3CStorageModule::Exemplar()->getModuleID(), 
              A3CStorageModule::strAdminAccessGroup))
	{
		Pace::A3CStorageModule::Exemplar()->UpdateLockedBaselines();
		IlvMessageLabel *pL=dynamic_cast<IlvMessageLabel*>(getObject("TXT_LookupTime"));
		if(pL)
			pL->setLabel(A3CProjectModule::Instance()->GetLookupTime().c_str());
		DoRefreshVersions();
		DoRuleRefreshVersions();
		OrgFillTree();
		CcgFillList();
		AcFillMtx();
		CustomerFillMtx();
		PRAdjustToolbar();
		moveToScreen();
		show();
	}
	else
	{
		TheApplication->getMainDial()->getFormAlert()->
				showAlert(PAFA_STOP,"&A3C access right not granted", NULL, NULL, NULL, "&APRJ_OK");
	}
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnClose(IlvGraphic* , IlvAny pDlg_)
{
	Exemplar()->hide();
}

/*static*/
void A3CAdminToolDlg::OnUploadUserData(IlvGraphic* , IlvAny )
{
	A3CStorageModule::Exemplar()->UploadPacelabData();
}

// cabin item baseline stuff

void A3CAdminToolDlg::BaselineUnderConstruction(bool commit_, const std::string& lockedBy_)
{
	assert(_pItmBLDefine && _pItmBLCommit);
	assert(!(commit_ && !lockedBy_.empty())); // if we have a baseline to commit, the database cannot be locked
	
	bool bRight = cabinlayout::PacePUMInterface::Exemplar().CanDo(
              A3CStorageModule::strCreateCabinItemBaselines, 
              A3CStorageModule::Exemplar()->getModuleID(), 
              A3CStorageModule::strAdminAccessGroup);
		
	if(_pItmBLDefine && _pItmBLCommit)
	{
		_pItmBLDefine->setSensitive(bRight && !commit_ && lockedBy_.empty());
		if(Pace::A3CStorageModule::Exemplar()->GetCurrentBaseline()==ADMINMODE && ProjectLoaded())
		{// do not commit/rollback as long as the loaded project uses the bl under construction
			_pItmBLCommit->setSensitive(IlFalse);
			_pItmBLRollback->setSensitive(IlFalse);
		}
		else
		{
			_pItmBLCommit->setSensitive(commit_ && lockedBy_.empty());
			_pItmBLRollback->setSensitive(commit_ && lockedBy_.empty());
		}
		
	}

	// set locking info
	if(!bRight)
	{
		_pItmBLDefine->setToolTip("&A3C access right not granted");
	}
	else if(!lockedBy_.empty())
	{std::string info;
	 		
		info = IlvGetMessage(TheApplication->getDisplay(), "&currently reserved by: ");
		info += lockedBy_;
		_pItmBLDefine->setToolTip(info.c_str());
		
	}
	else
	{
		_pItmBLDefine->setToolTip("&A3C define new baseline");
	}

	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_Version");
	assert(pCB);
	if(pCB)
	{
		pCB->setSensitive(!ProjectLoaded());
		pCB->reDraw();
	}
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnDefineBaseline(IlvGraphic* , IlvAny)
{
    static ATNewCIBaselineDlg* pDlg=NULL;

	if(!pDlg)
		pDlg=new ATNewCIBaselineDlg();
	pDlg->moveToScreen();
	pDlg->showModal();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnCommitBaseline(IlvGraphic* , IlvAny)
{
	int vers = Pace::A3CStorageModule::Exemplar()->GetCurrentBaseline();
	if(vers==ADMINMODE)
		vers=LATEST;

	if(!Pace::A3CStorageModule::Exemplar()->CommitBaseline())
		assert(0);
	Pace::A3CStorageModule::Exemplar()->InitRepository(vers);
	A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
	A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();

}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRollbackBaseline(IlvGraphic* , IlvAny)
{
	if(!Pace::A3CStorageModule::Exemplar()->RollbackBaseline())
		assert(0);
	A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
	A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();
	
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnShowDiffOnly(IlvGraphic* , IlvAny)
{
	Exemplar()->ToggleShowDiff();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnExpandItem(IlvGraphic* , IlvAny storagename)
{
	Exemplar()->ExpandItem();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnShowItem(IlvGraphic* , IlvAny )
{
	Exemplar()->ShowItem();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRefreshVersions(IlvGraphic* , IlvAny pDlg_)
{
	Exemplar()->DoRefreshVersions();
}

void A3CAdminToolDlg::DoRefreshVersions()
{
	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_Version");
	Pace::A3CStorageModule::BASELINE_MAP::const_iterator it;

	pCB->empty();
	bool locked;
	std::string lockedBy;
	const Pace::A3CStorageModule::BASELINE_MAP& map = Pace::A3CStorageModule::Exemplar()->GetBaselines(locked, &lockedBy);

	// adjust toolbar
	BaselineUnderConstruction( map.find(ADMINMODE)!=map.end(), lockedBy ); // if there is an ID_Baseline -1 (Adminmode) we have one under construction

	// fill combo box
	for(it=map.begin(); it!=map.end(); it++)
	{
		pCB->addLabel(it->second.c_str());
	}
	pCB->setLabel(Pace::A3CStorageModule::Exemplar()->GetStrCurrentBaseline().c_str());

	FillTree();
	

}

void A3CAdminToolDlg::FillTree()
{
	assert(_pCITree);
	if ( !_pCITree )
		return;
	_pCITree->removeAllItems();

	PaceBaseAttributeManager *pBAMgr = TheApplication->getBaseAttributeManager();
	if ( !pBAMgr )
		return;
	cabinlayout::StorageAccessMgr *pSAMgr = cabinlayout::StorageAccessMgr::Exemplar();
	if ( !pSAMgr )
		return;
	Pace::A3CStorageModule *pStoMod = Pace::A3CStorageModule::Exemplar();
	if ( !pStoMod )
		return;

	// get all storable objects and put accessors into tree structure
	std::list<const cabinlayout::StorageAccessor *> lstAcc = pSAMgr->GetAccessors();
	std::list<const cabinlayout::StorageAccessor *>::const_iterator citAcc;
	IlvTreeGadgetItem *pCINodeItm = _pCITree->addItem(NULL, "&CL_CabinItem");
	IlvTreeGadgetItem *pACNodeItm = _pCITree->addItem(NULL, "&APRJ_Aircraft");
	for (citAcc=lstAcc.begin(); citAcc!=lstAcc.end(); ++citAcc)
	{
		const cabinlayout::StorageAccessor *pAcc = *citAcc;
		int id = pStoMod->GetClassID(pAcc->StorageClassName());
		assert(id != NULL_ID);
		if (id == NULL_ID)
			continue;
	
		// distinguish between different types of objects:
		bool bIsCabinItem = (dynamic_cast<const cabinlayout::CabinItemStorageAccessor *>(pAcc) != NULL);

		// insert the treeitem
		IlvBitmap *pBitmap = TheApplication->getBitmapVault()->getIlvBitmap(pAcc->IconName().c_str());
		IlvTreeGadgetItem *pItm = _pCITree->addItem((bIsCabinItem ? pCINodeItm : pACNodeItm), 
																								pAcc->DBMDialogTitle().c_str(), 
																								-1, 
																								pBitmap);
		 
		// evaluate if changes of this item are resident in the current baseline
		bool bDiffer = false;
		if (_BLShowDiff)
		{
			boost::shared_ptr<A3CStorage::CI::ClassDef> pCD = pStoMod->GetClassDef(id);
			std::map< std::string,boost::shared_ptr<CI::ClassInst> >::iterator itStrID = pCD->_inst_id.begin();
			while (!bDiffer && itStrID!=pCD->_inst_id.end())
				bDiffer = ((itStrID++)->second->_OriginBaselineID == pStoMod->GetCurrentBaseline());
		}
		else
			bDiffer = !pStoMod->GetClassDef(id)->_inst_id.empty();

		// add client data if any changes exist
		pItm->setUnknownChildCount(bDiffer ? IlTrue : IlvFalse);
		pItm->setClientData(bDiffer ? (IlAny)pAcc : NULL);
	} // for (all accessors) ...

	_pCITree->sort();
}

void A3CAdminToolDlg::ExpandItem()
{
	IlvTreeGadgetItem *pItm = _pCITree->getCallbackItem();
	const cabinlayout::StorageAccessor *pAcc = (const cabinlayout::StorageAccessor *)(pItm->getClientData());
	if ( !pAcc ) // not populated
		return;
  pItm->setClientData(NULL);

	Pace::A3CStorageModule *pStoMod = Pace::A3CStorageModule::Exemplar();
	if ( !pStoMod )
		return;
		
	int id = pStoMod->GetClassID(pAcc->StorageClassName());
	boost::shared_ptr<A3CStorage::CI::ClassDef> pCD = pStoMod->GetClassDef(id);

	// retrieve all storage values
	std::vector<std::string> vecMissingParam;
	std::auto_ptr<PaceStorable> pStorable = pAcc->GetStorable();
	PaceStorageData *pSD = pStorable->ClassData();
	if ( !pSD )
		return;

	const PaceStorageValues lstSV = pSD->Values();
	delete pSD;
	if (lstSV.size() == 0)
		return;

	// retrieve any key values
	PaceStorageValues::const_iterator citSV;
	for (citSV=lstSV.begin(); citSV!=lstSV.end(); ++citSV)
		if (		citSV->IsKey() 
				&& (	citSV->Type()==PaceStorageValue::string_type
						||citSV->Type()==PaceStorageValue::text_type)	)
			vecMissingParam.push_back(citSV->Name());
	pStoMod->ReadMissingParam(pCD, vecMissingParam);

	std::map< std::string,boost::shared_ptr<CI::ClassInst> >::iterator itStrID;
	for (itStrID=pCD->_inst_id.begin(); itStrID!=pCD->_inst_id.end(); ++itStrID)
	{
		if (_BLShowDiff && itStrID->second->_OriginBaselineID!=pStoMod->GetCurrentBaseline())
			continue;

		// evaluate name of item, baseline version
		std::stringstream ssName;

		std::vector<std::string>::const_iterator itKeyValues;
		for (itKeyValues=vecMissingParam.begin(); itKeyValues!=vecMissingParam.end(); ++itKeyValues)
		{
			std::map< std::string,boost::shared_ptr<CI::Param> >::iterator itParam;
			itParam = itStrID->second->_param_name.find(*itKeyValues);

			if (itKeyValues !=vecMissingParam.begin() )
				ssName << ", ";

			if (itParam != itStrID->second->_param_name.end())
			{
				ssName << itParam->second->_ValueStr;
				
				if (itKeyValues == vecMissingParam.end()-1)
				{
					ssName << " (";
					if (itStrID->second->_VersionID > 0)
						ssName << itStrID->second->_VersionID;
					else
						ssName << (pStoMod->GetCurrentBaseline()==ADMINMODE ? "*" : IlvGetMessage(TheApplication->getDisplay(),"&local"));
					ssName << ")";
				}
			}
			else
			{
				ssName << "name?";
			}
		}

		// insert new item
		IlvTreeGadgetItem *pChItm = _pCITree->addItem(pItm, ssName.str().c_str());
		pChItm->setClientData((IlAny)pAcc);
	}

	pItm->sortChildren();
}

void A3CAdminToolDlg::ShowItem()
{
	IlvTreeGadgetItem *pItm = _pCITree->getCallbackItem();
	const cabinlayout::StorageAccessor *pAcc = (const cabinlayout::StorageAccessor *)(pItm->getClientData());
	if ( !pAcc ) // not populated
		return;
	Pace::A3CStorageModule *pStoMod = Pace::A3CStorageModule::Exemplar();
	if ( !pStoMod )
		return;

	// distinguish between different types of objects:
	const cabinlayout::CabinItemStorageAccessor *pCIAcc = dynamic_cast<const cabinlayout::CabinItemStorageAccessor *>(pAcc);
	if ( !pCIAcc )
		return;

	cabinlayout::PaceCabinItemBA *pCIBA = pCIAcc->GetBA();
	assert(pCIBA);
	if ( !pCIBA )
		return;

	std::string lbl		= pItm->getLabel();
	std::string name = lbl.substr(0, lbl.find_last_of(' '));
	int id = pStoMod->GetClassID(pCIAcc->StorageClassName());
	boost::shared_ptr<PaceStorageData> psd = pStoMod->GetClassData(id);
	PaceStorageValue v = psd->Value("name");
	PaceStorageValue vk(v.Name(), v.Type(), true);//must be key
	vk.SetValue(name);
	psd->SetValue(vk);
	std::list<PaceStorageData*> lstFound;
	pStoMod->Find(psd.get(), lstFound);
	assert(lstFound.size() == 1);
	if (lstFound.size() == 1)
	{
		PaceStorageData *pFound = *(lstFound.begin());
		cabinlayout::PaceDataDialogCabinItem *const pDialog = pCIBA->GetDataFrontendDialog();
		if (pDialog)  
			pDialog->showDial(pFound, pCIBA, true);
			// baseline could have been changed now
			FillTree();
			delete pFound;
	}
}

void A3CAdminToolDlg::ToggleShowDiff()
{
	_BLShowDiff=!_BLShowDiff;
	FillTree();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnVersionSelect(IlvGraphic* , IlvAny pDlg_)
{
	Exemplar()->SetVersion();

}

void A3CAdminToolDlg::SetVersion()
{

	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_Version");	
	std::string vers;

	vers=pCB->getSelection();
	Pace::A3CStorageModule::Exemplar()->SetBaseline(vers);
	FillTree();
	
}

// end cabin item stuff

// rule baseline stuff

void A3CAdminToolDlg::RuleBaselineUnderConstruction(bool commit_, const std::string& lockedBy_)
{
	assert(_pItmRuleBLDefine && _pItmRuleBLCommit);
	assert(!(commit_ && !lockedBy_.empty())); // if we have a baseline to commit, the database cannot be locked
	
	bool bRight = cabinlayout::PacePUMInterface::Exemplar().CanDo(
              A3CStorageModule::strCreateRuleBaselines, 
			  A3CStorageModule::Exemplar()->getModuleID(), 
              A3CStorageModule::strAdminAccessGroup);

	if(_pItmRuleBLDefine && _pItmRuleBLCommit)
	{
		_pItmRuleBLDefine->setSensitive(bRight && !commit_ && lockedBy_.empty());
		if(Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline()==ADMINMODE && ProjectLoaded())
		{// do not commit/rollback as long as the loaded project uses the bl under construction
			_pItmRuleBLCommit->setSensitive(IlFalse);
			_pItmRuleBLRollback->setSensitive(IlFalse);
		}
		else
		{
			_pItmRuleBLCommit->setSensitive(commit_ && lockedBy_.empty());
			_pItmRuleBLRollback->setSensitive(commit_ && lockedBy_.empty());
		}
		
	}

	// set locking info
	if(!bRight)
	{
		_pItmRuleBLDefine->setToolTip("&A3C access right not granted");
	}
	else if(!lockedBy_.empty())
	{std::string info;
	 		
		info = IlvGetMessage(TheApplication->getDisplay(), "&currently reserved by: ");
		info += lockedBy_;
		_pItmRuleBLDefine->setToolTip(info.c_str());
		
	}
	else
	{
		_pItmRuleBLDefine->setToolTip("&A3C define new baseline");
	}

	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_RuleVersion");
	assert(pCB);
	if(pCB)
	{
		pCB->setSensitive(!ProjectLoaded());
		pCB->reDraw();
	}
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleDefineBaseline(IlvGraphic* , IlvAny)
{static ATNewRLBaselineDlg* pDlg=NULL;

	if(!pDlg)
		pDlg=new ATNewRLBaselineDlg();
	pDlg->moveToScreen();
	pDlg->showModal();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleCommitBaseline(IlvGraphic* , IlvAny)
{
	int vers = Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline();
	if(vers==ADMINMODE)
		vers=LATEST;

	if(!Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->CommitBaseline())
		assert(0);
	Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->InitRepository(vers);
	A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
	A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();
	
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleRollbackBaseline(IlvGraphic* , IlvAny)
{
	if(!A3CStorageModule::Exemplar(A3CStorageModule::RULE)->RollbackBaseline())
		assert(0);
	A3CAdminToolDlg::Exemplar()->DoRuleRefreshVersions();
	A3CAdminToolDlg::Exemplar()->DoRefreshVersions();
	
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleShowDiffOnly(IlvGraphic* , IlvAny)
{
	Exemplar()->RuleToggleShowDiff();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleExpandItem(IlvGraphic* , IlvAny storagename)
{
	Exemplar()->RuleExpandItem();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleShowItem(IlvGraphic* , IlvAny )
{
	Exemplar()->RuleShowItem();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleRefreshVersions(IlvGraphic* , IlvAny pDlg_)
{
	Exemplar()->DoRuleRefreshVersions();
}

void A3CAdminToolDlg::DoRuleRefreshVersions()
{
	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_RuleVersion");
	A3CStorageModule::BASELINE_MAP::const_iterator it;

	pCB->empty();
	bool locked;
	std::string lockedBy;
	const A3CStorageModule::BASELINE_MAP& map = A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetBaselines(locked, &lockedBy);

	// adjust toolbar
	RuleBaselineUnderConstruction( map.find(ADMINMODE)!=map.end(), lockedBy ); // if there is an ID_Baseline -1 (Adminmode) we have one under construction

	// fill combo box
	for(it=map.begin(); it!=map.end(); it++)
	{
		pCB->addLabel(it->second.c_str(), (IlAny)it->first);
	}
	pCB->setLabel(A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetStrCurrentBaseline().c_str());

	RuleFillTree();
}

void A3CAdminToolDlg::RuleFillTree()
{
	assert(_pRLTree);
	if(_pRLTree)
	{std::map<int, boost::shared_ptr<A3CStorage::CI::ClassDef> >& repo = A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetRepository();
	 std::map<int, boost::shared_ptr<A3CStorage::CI::ClassDef> >::const_iterator rit;
	 int FolderSetID = A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetClassID("PaceFolderSet");
	 IlvBitmap* pRulebaseBMP = TheApplication->getBitmapVault()->getIlvBitmap("RuleTree");
	 IlvBitmap* pTopFolderBMP = TheApplication->getBitmapVault()->getIlvBitmap("PaceToplevelFolder");
	 
		_pRLTree->removeAllItems();
		_treeitm.clear();
		if(FolderSetID!=NULL_ID)
		{
			rit = repo.find(FolderSetID);
			if(rit!= repo.end())
			{IlvTreeGadgetItem* pRootItm;
			 std::map<std::string, boost::shared_ptr<CI::ClassInst> >::iterator iit;

				for(iit=rit->second->_inst_id.begin();iit!=rit->second->_inst_id.end(); iit++)
				{std::map<std::string , boost::shared_ptr<CI::Param> >::const_iterator pit;
					
					pit=iit->second->_param_name.find("name");
					assert(pit!=iit->second->_param_name.end());
					if(pit==iit->second->_param_name.end())
					{
						continue;
					}
					std::stringstream lbl;
					lbl << pit->second->_ValueStr << " (";
					if(iit->second->_VersionID>0)
					{
						lbl << iit->second->_VersionID;
					}
					else
					{
						if(Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline() == ADMINMODE )
						{
							lbl << "*";
						}
						else
						{
							lbl << IlvGetMessage(TheApplication->getDisplay(), "&local");
						}
					}
					lbl << ")";
					pRootItm=_pRLTree->addItem(0, lbl.str().c_str(), -1, pRulebaseBMP);
					if(pRootItm)
					{
						_treeitm[pRootItm]=iit->second;
						pit=iit->second->_param_name.find("content");
						assert(pit!=iit->second->_param_name.end());
						for(int i=0; pit!=iit->second->_param_name.end() && i < pit->second->_ValueArray.size(); i++)
						{std::string s;
						 IlvTreeGadgetItem* pItm;
						 std::list<boost::shared_ptr<A3CStorage::CI::ClassInst> > lstFoundObjects;
						 
							if( pit->second->_ValueArray[i].size() > 2 ) // has to be "|xyz|=0"
							{
								s=pit->second->_ValueArray[i].substr(1);
								s=s.substr(0, s.rfind('|'));
								PaceStorageData sd("PaceToplevelFolderOA");
								PaceStorageValue val("name", PaceStorageValue::string_type);
								val.SetValue(s, PaceStorageValue::restrict_equal);
								sd.Values().push_back(val);
								A3CStorageModule::Exemplar(A3CStorageModule::RULE)->Find(&sd, lstFoundObjects);
								if(!lstFoundObjects.empty())
								{
									if(_RuleBLShowDiff && (*lstFoundObjects.begin())->_OriginBaselineID!=A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline())
									{// show only differences, and it's not originated in the current..

										continue;
									}
									std::stringstream lbl;
									lbl << s << " (";
									if((*lstFoundObjects.begin())->_VersionID>0)
									{
										lbl << (*lstFoundObjects.begin())->_VersionID;
									}
									else
									{
										if(Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline() == ADMINMODE )
										{
											lbl << "*";
										}
										else
										{
											lbl << IlvGetMessage(TheApplication->getDisplay(), "&local");
										}
									}
									lbl << ")";
									assert(lstFoundObjects.size()==1);
									pItm=_pRLTree->addItem(pRootItm, lbl.str().c_str(), -1, pTopFolderBMP);
									_treeitm[pItm]=*lstFoundObjects.begin();
									pItm->setUnknownChildCount(IlTrue);
                                    pItm->setClientData((IlAny)1); //still to be populated
								}
							}
						}
					}
				}
			}
			_pRLTree->sort();
		}
	}
}

void A3CAdminToolDlg::FillRuleFolder(IlvTreeGadgetItem* pItm_, boost::shared_ptr<A3CStorage::CI::ClassInst> rf_)
{
	assert(_pRLTree);
	if(_pRLTree)
	{std::map<std::string , boost::shared_ptr<CI::Param> >::const_iterator pit;
	 IlvBitmap* pFolderBMP = TheApplication->getBitmapVault()->getIlvBitmap("PaceRuleFolder");
	 IlvBitmap* pRuleBMP = TheApplication->getBitmapVault()->getIlvBitmap("PaceRule");
				
		pit=rf_->_param_name.find("rulenodeChildren");
		assert(pit!=rf_->_param_name.end());
		if(pit!=rf_->_param_name.end())
		{
			assert(pit->second->_ValueArray.size() % 2 == 0); // always name and versionSubcode per entry
			for(int i=0; i < pit->second->_ValueArray.size(); i++)
			{std::string name, s;

				IlvTreeGadgetItem* pItm;
				std::list<boost::shared_ptr<A3CStorage::CI::ClassInst> > lstFoundObjects;
				
				if( pit->second->_ValueArray[i].size() > 2 ) // has to be "|xyz|"
				{
					name=pit->second->_ValueArray[i].substr(1);
					name=name.substr(0, name.rfind('|'));
					PaceStorageData sd1("PaceRuleOA");
					PaceStorageData sd2("PaceRuleFolderOA");
					PaceStorageValue val("name", PaceStorageValue::string_type);
					val.SetValue(name, PaceStorageValue::restrict_equal);
					sd1.Values().push_back(val);
					sd2.Values().push_back(val);
					i++;
					assert( pit->second->_ValueArray[i].size() > 2 ); // has to be "|bla~|bli|"
					s=pit->second->_ValueArray[i].substr(1);
					s=s.substr(0, s.rfind('|'));
					for(std::string::size_type pos=s.find("~|"); pos!=std::string::npos; pos=s.find("~|", pos ) )
					{
						s.replace(pos, 2, "|");
					}
					PaceStorageValue val2("versionSubcode", PaceStorageValue::string_type);
					val2.SetValue(s, PaceStorageValue::restrict_equal);
					sd1.Values().push_back(val2);
					sd2.Values().push_back(val2);
					A3CStorageModule::Exemplar(A3CStorageModule::RULE)->Find(&sd1, lstFoundObjects);
					if(!lstFoundObjects.empty())
					{
						// rule
						assert(lstFoundObjects.size()==1);
						std::stringstream lbl;
						lbl << name << " (";
						if((*lstFoundObjects.begin())->_VersionID>0)
						{
							lbl << (*lstFoundObjects.begin())->_VersionID;
						}
						else
						{
							if(Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline() == ADMINMODE )
							{
								lbl << "*";
							}
							else
							{
								lbl << IlvGetMessage(TheApplication->getDisplay(), "&local");
							}
						}
						lbl << ")";
						pItm=_pRLTree->addItem(pItm_, lbl.str().c_str(), -1, pRuleBMP);
						_treeitm[pItm]=*lstFoundObjects.begin();
					}
					else
					{
						A3CStorageModule::Exemplar(A3CStorageModule::RULE)->Find(&sd2, lstFoundObjects);
						if(!lstFoundObjects.empty())
						{
							// rule folder
							assert(lstFoundObjects.size()==1);
							std::stringstream lbl;
							lbl << name << " (";
							if((*lstFoundObjects.begin())->_VersionID>0)
							{
								lbl << (*lstFoundObjects.begin())->_VersionID;
							}
							else
							{
								if(Pace::A3CStorageModule::Exemplar(A3CStorageModule::RULE)->GetCurrentBaseline() == ADMINMODE )
								{
									lbl << "*";
								}
								else
								{
									lbl << IlvGetMessage(TheApplication->getDisplay(), "&local");
								}
							}
							lbl << ")";
							pItm=_pRLTree->addItem(pItm_, lbl.str().c_str(), -1, pFolderBMP);
							_treeitm[pItm]=*lstFoundObjects.begin();
							FillRuleFolder(pItm, *lstFoundObjects.begin());
						}
					}
				}
			}
			pItm_->sortChildren();
		}
	}
}

void A3CAdminToolDlg::RuleExpandItem()
{
	IlvTreeGadgetItem* pItm=_pRLTree->getCallbackItem();
	void* data=pItm->getClientData();
	if(data) // not populated
	{boost::shared_ptr<A3CStorage::CI::ClassInst> rf;
		pItm->setClientData((IlAny)0);
		assert(_treeitm.find(pItm)!=_treeitm.end());
		rf=_treeitm[pItm];
		FillRuleFolder(pItm, rf);
	}
}

void A3CAdminToolDlg::RuleShowItem()
{
	// nothing is shown for rules..
}

void A3CAdminToolDlg::RuleToggleShowDiff()
{
	_RuleBLShowDiff=!_RuleBLShowDiff;
	RuleFillTree();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnRuleVersionSelect(IlvGraphic* , IlvAny pDlg_)
{
	Exemplar()->RuleSetVersion();

}

void A3CAdminToolDlg::RuleSetVersion()
{

	IlvScrolledComboBox* pCB=(IlvScrolledComboBox*)getObject("CBX_RuleVersion");	
	int bl = (int) pCB->getItemData((IlUShort)pCB->getFirstSelectedItem());
	std::set<std::string> ruleclasses;
	ruleclasses.insert("PaceRuleOA");
	ruleclasses.insert("PaceRuleFolderOA");
	ruleclasses.insert("PaceRulebaseDefault");
	ruleclasses.insert("PaceToplevelFolderOA");
	ruleclasses.insert("PaceFolderSet");
	Pace::A3CStorageModule::Exemplar(Pace::A3CStorageModule::RULE)->InitRepository(bl, ruleclasses);
	RuleFillTree();
	
}

// end rule stuff

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnPREnterEdit(IlvGraphic* , IlvAny )
{
	A3CProjectModule::Instance()->EnterEditMode();
	Exemplar()->OrgFillTree();
	Exemplar()->CcgFillList();
	Exemplar()->AcFillMtx();
	Exemplar()->CustomerFillMtx();
	Exemplar()->PRAdjustToolbar();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnPRUpload(IlvGraphic* , IlvAny )
{
	A3CProjectModule::Instance()->UploadChanges();
	Exemplar()->PRAdjustToolbar();
}

void A3CAdminToolDlg::PRAdjustToolbar()
{std::string who;
 std::string info;

	_editMode=A3CProjectModule::Instance()->IsInEditMode(who);
	assert(	_pItmPREnterEditCust &&
			_pItmPRUploadCust &&
			_pItmPREnterEditOrg &&
			_pItmPRUploadOrg &&
			_pBtnUploadUser);
	
	if(	_pItmPREnterEditCust &&
		_pItmPRUploadCust &&
		_pItmPREnterEditOrg &&
		_pItmPRUploadOrg &&
		_pBtnUploadUser)
	{
		bool bRight = cabinlayout::PacePUMInterface::Exemplar().CanDo(
              A3CStorageModule::strEnterEditMode, 
              A3CStorageModule::Exemplar()->getModuleID(), 
              A3CStorageModule::strAdminAccessGroup);
		
		if(!bRight)
		{
			info = "&A3C access right not granted";
		}
		else if(!_editMode && !who.empty())
		{	 		
			info = IlvGetMessage(TheApplication->getDisplay(), "&currently reserved by: ");
			info += who;
			
			
		}
		else
		{
			info = "&A3C enter edit mode";
		}
		
		_pItmPREnterEditCust->setSensitive(bRight && !_editMode && who.empty());
		_pItmPREnterEditCust->setToolTip(info.c_str());
		_pItmPRUploadCust->setSensitive(_editMode);
		
		_pItmPREnterEditOrg->setSensitive(bRight && !_editMode && who.empty());
		_pItmPREnterEditOrg->setToolTip(info.c_str());
		_pItmPRUploadOrg->setSensitive(_editMode);
		
		bRight=cabinlayout::PacePUMInterface::Exemplar().CanDo(
              A3CStorageModule::strUploadUserData, 
              A3CStorageModule::Exemplar()->getModuleID(), 
              A3CStorageModule::strAdminAccessGroup);

		_pBtnUploadUser->setSensitive(_editMode && bRight);
		if(!bRight)
		{
			IlvToolTip* pToolTip = new IlvToolTip("&A3C access right not granted");
			_pBtnUploadUser->setNamedProperty(pToolTip);
		}
		else
		{
			IlvToolTip* pToolTip = new IlvToolTip("&A3C Upload User Data");
			_pBtnUploadUser->setNamedProperty(pToolTip);
		}
		_pBtnUploadUser->reDraw();

		OrgSelect();
		CcgSelect();
		AcSelect();
		CustomerSelect();
	}
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnOrgSelect(IlvGraphic* , IlvAny )
{
	Exemplar()->OrgSelect();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnOrgAddSibling(IlvGraphic* , IlvAny )
{
	Exemplar()->OrgAddSibling();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnOrgAddChild(IlvGraphic* , IlvAny )
{
	Exemplar()->OrgAddChild();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnOrgDeleteEntry(IlvGraphic* , IlvAny )
{
	Exemplar()->OrgDeleteEntry();
}

/*static*/
void ILVCALLBACK A3CAdminToolDlg::OnOrgRename(IlvGraphic* , IlvAny )
{
	Exemplar()->OrgRename();
}

void A3CAdminToolDlg::OrgFillTree()
{std::list<const boost::shared_ptr<PR::Organisation> > org;
 std::list<const boost::shared_ptr<PR::Organisation> >::iterator it;
 std::map<int, IlvTreeGadgetItem*> imap;

	
	assert(	_pOrgTree &&
			_pItmOrgAddSib &&
			_pItmOrgAddChi &&
			_pItmOrgDel );
	if(	_pOrgTree &&
		_pItmOrgAddSib &&
		_pItmOrgAddChi &&
		_pItmOrgDel )
	{IlvTreeGadgetItem *pItm, *pPar;

		_pOrgTree->removeAllItems();
		IlvBitmap* aib = TheApplication->getDisplay()->getBitmap("Resource/Modules/A3CStorage/Icons/Logo Airbus.png", IlTrue);
		if(aib)
		{
			pPar=_pOrgTree->addItem(0, "", -1, aib);
		}
		else
		{
			pPar=_pOrgTree->addItem(0, "Airbus");
		}
		pPar->setClientData((IlAny)NULL_ID);

		A3CProjectModule::Instance()->getOrg(PR::FAM, org);
		for(it=org.begin(); it!=org.end(); it++)
		{
			pItm=_pOrgTree->addItem(pPar, (*it)->getName().c_str());
			pItm->setClientData((IlAny)(*it)->getOrgNodeID());
			imap[(*it)->getOrgNodeID()]=pItm;
		}
		A3CProjectModule::Instance()->getOrg(PR::TYP, org);
		for(it=org.begin(); it!=org.end(); it++)
		{
			pPar=imap[(*it)->getParent()->getOrgNodeID()];
			pItm=_pOrgTree->addItem(pPar, (*it)->getName().c_str());
			pItm->setClientData((IlAny)(*it)->getOrgNodeID());
			imap[(*it)->getOrgNodeID()]=pItm;
		}
		A3CProjectModule::Instance()->getOrg(PR::SER, org);
		for(it=org.begin(); it!=org.end(); it++)
		{
			pPar=imap[(*it)->getParent()->getOrgNodeID()];
			pItm=_pOrgTree->addItem(pPar, (*it)->getName().c_str());
			pItm->setClientData((IlAny)(*it)->getOrgNodeID());
			imap[(*it)->getOrgNodeID()]=pItm;
		}
		A3CProjectModule::Instance()->getOrg(PR::SSP, org);
		for(it=org.begin(); it!=org.end(); it++)
		{
			pPar=imap[(*it)->getParent()->getOrgNodeID()];
			pItm=_pOrgTree->addItem(pPar, (*it)->getName().c_str());
			pItm->setClientData((IlAny)(*it)->getOrgNodeID());
			imap[(*it)->getOrgNodeID()]=pItm;
		}
		
		_pItmOrgAddSib->setSensitive(IlFalse);
		_pItmOrgAddChi->setSensitive(IlFalse);
		_pItmOrgDel->setSensitive(IlFalse);
	}
}

void A3CAdminToolDlg::OrgSelect()
{
	_pOrgTree->setEditable(_editMode);
	IlvTreeGadgetItem* pItm=_pOrgTree->getFirstSelectedItem();
	if(pItm)
	{boost::shared_ptr<PR::Organisation> pOrg;

		int nodeID = (int)pItm->getClientData();
		if(_editMode && nodeID!=NULL_ID)
		{
			A3CProjectModule::Instance()->getOrgNode(nodeID, pOrg);

			_pItmOrgAddSib->setSensitive(IlTrue);
			_pItmOrgAddChi->setSensitive(pOrg->getLevel()!=PR::SSP ? IlTrue:IlFalse);
			_pItmOrgDel->setSensitive(IlTrue);
		}
		else
		{
			_pItmOrgAddSib->setSensitive(IlFalse);
			_pItmOrgAddChi->setSensitive(IlFalse);
			_pItmOrgDel->setSensitive(IlFalse);
		}
	
	}
}

void A3CAdminToolDlg::OrgAddSibling()
{
	IlvTreeGadgetItem* pItm=_pOrgTree->getFirstSelectedItem();
	if(pItm)
	{boost::shared_ptr<PR::Organisation> pOrg;
	 IlvTreeGadgetItem* pPar;

		int nodeID = (int)pItm->getClientData();
		A3CProjectModule::Instance()->getOrgNode(nodeID, pOrg);
		PR::Organisation newOrg(*(pOrg.get()));
		switch(pOrg->getLevel())
		{
		case PR::FAM:
			newOrg.setName("<new AC Family>");
			break;
		case PR::TYP:
			newOrg.setName("<new AC Type>");
			break;
		case PR::SER:
			newOrg.setName("<new AC Series>");
			break;
		case PR::SSP:
			newOrg.setName("<new Standard Specification>");
			break;
		default:
			newOrg.setName("<new unknown>");
			break;
		}
		newOrg.setOrgNodeID(NULL_ID);
		if(A3CProjectModule::Instance()->createOrgNode(newOrg))
		{
			pPar=pItm->getParent();
			pItm=_pOrgTree->addItem(pPar, newOrg.getName().c_str());
			pItm->setClientData((IlAny)newOrg.getOrgNodeID());
		}
	}
}

void A3CAdminToolDlg::OrgAddChild()
{
	IlvTreeGadgetItem* pItm=_pOrgTree->getFirstSelectedItem();
	if(pItm)
	{boost::shared_ptr<PR::Organisation> pOrg;
	 
		int nodeID = (int)pItm->getClientData();
		A3CProjectModule::Instance()->getOrgNode(nodeID, pOrg);
		PR::Organisation newOrg(*(pOrg.get()));
		switch(pOrg->getLevel())
		{
		case PR::FAM:
			newOrg.setName("<new AC Type>");
			newOrg.setLevel(PR::TYP);
			break;
		case PR::TYP:
			newOrg.setName("<new AC Series>");
			newOrg.setLevel(PR::SER);
			break;
		case PR::SER:
			newOrg.setName("<new Standard Specification>");
			newOrg.setLevel(PR::SSP);
			break;
		default:
			newOrg.setName("<new unknown>");
			break;
		}
		newOrg.setOrgNodeID(NULL_ID);
		newOrg.setParent(pOrg);
		if(A3CProjectModule::Instance()->createOrgNode(newOrg))
		{
			pItm=_pOrgTree->addItem(pItm, newOrg.getName().c_str());
			pItm->setClientData((IlAny)newOrg.getOrgNodeID());
		}

			
	}

}

void A3CAdminToolDlg::OrgDeleteEntry()
{
	IlvTreeGadgetItem* pItm=_pOrgTree->getFirstSelectedItem();
	if(pItm)
	{boost::shared_ptr<PR::Organisation> pOrg;
	 
		int nodeID = (int)pItm->getClientData();
		if(nodeID!=NULL_ID && A3CProjectModule::Instance()->deleteOrgNode(nodeID))
		{
			OrgFillTree();
		}
	}
}

void A3CAdminToolDlg::OrgRename()
{
	IlvTreeGadgetItem* pItm=_pOrgTree->getFirstSelectedItem();
	if(pItm)
	{boost::shared_ptr<PR::Organisation> pOrg;
	 
		int nodeID = (int)pItm->getClientData();
		std::string name = pItm->getLabel();
		if(nodeID==NULL_ID)
		{
			pItm->setLabel("");
		}
		else if(!A3CProjectModule::Instance()->renameOrgNode(nodeID, name))
		{
			A3CProjectModule::Instance()->getOrgNode(nodeID, pOrg);
			pItm->setLabel(pOrg->getName().c_str());
			pItm->reDraw();
		}
	}
}

/*static*/
void A3CAdminToolDlg::OnCcgSelect(IlvGraphic* , IlvAny )
{
	Exemplar()->CcgSelect();
}
/*static*/
void A3CAdminToolDlg::OnCcgNew(IlvGraphic* , IlvAny )
{
	Exemplar()->CcgNew();
}
/*static*/
void A3CAdminToolDlg::OnCcgRename(IlvGraphic* , IlvAny )
{
	Exemplar()->CcgRename();
}
/*static*/
void A3CAdminToolDlg::OnCcgDelete(IlvGraphic* , IlvAny )
{
	Exemplar()->CcgDelete();
}

void A3CAdminToolDlg::CcgFillList()
{
	if(_pCcgList)
	{
		_pCcgList->empty();
		const std::map<int/*CcgID*/, boost::shared_ptr<PR::CCG> >& repo_ccg = A3CProjectModule::Instance()->getCCGs();
		std::map<int/*CcgID*/, boost::shared_ptr<PR::CCG> >::const_iterator cit;

		for(cit=repo_ccg.begin(); cit!=repo_ccg.end(); cit++)
		{
			_pCcgList->addLabel(cit->second->getName().c_str(), (IlAny)cit->second->getCcgID() );
		}
	}

}
void A3CAdminToolDlg::CcgSelect()
{
	if(_pCcgList->getFirstSelectedItem()!=-1)
	{
		_pItmCcgDel->setSensitive(_editMode);
	}
	else
	{
		_pItmCcgDel->setSensitive(IlFalse);
	}
	_pItmCcgNew->setSensitive(_editMode);
	_pCcgList->setEditable(_editMode);

}
void A3CAdminToolDlg::CcgNew()
{PR::CCG ccg;
	
	ccg.setName("<new Cabin Configuration Guide>");
	if(A3CProjectModule::Instance()->createCCG(ccg))
	{
		_pCcgList->addLabel(ccg.getName().c_str(), (IlAny)ccg.getCcgID() );
		_pCcgList->reDraw();
	}
}

void A3CAdminToolDlg::CcgRename()
{
	assert(_editMode);

	IlShort ndx = _pCcgList->getFirstSelectedItem();
	int CcgId=(int)_pCcgList->getItemData(ndx);
	std::string name=_pCcgList->getLabel(ndx);
	if(!A3CProjectModule::Instance()->renameCCG(CcgId, name))
	{boost::shared_ptr<PR::CCG> ccg;

		A3CProjectModule::Instance()->getCCG(CcgId, ccg);
		_pCcgList->setLabel(ndx, ccg->getName().c_str());	
	}
}

void A3CAdminToolDlg::CcgDelete()
{
	IlShort ndx = _pCcgList->getFirstSelectedItem();
	int CcgId=(int)_pCcgList->getItemData(ndx);
	if(A3CProjectModule::Instance()->deleteCCG(CcgId))
	{
		_pCcgList->removeLabel(ndx);
		CcgSelect();
	}
}

/*static*/
void A3CAdminToolDlg::OnCustomerSelect(IlvGraphic* , IlvAny )
{
	Exemplar()->CustomerSelect();
}
/*static*/
void A3CAdminToolDlg::OnCustomerNew(IlvGraphic* , IlvAny )
{
	Exemplar()->CustomerNew();
}
/*static*/
void A3CAdminToolDlg::OnCustomerChange(IlvGraphic* , IlvAny )
{
	Exemplar()->CustomerChange();
}
/*static*/
void A3CAdminToolDlg::OnCustomerRename(IlvGraphic* , IlvAny )
{
	Exemplar()->CustomerRename();
}
/*static*/
void A3CAdminToolDlg::OnCustomerDelete(IlvGraphic* , IlvAny )
{
	Exemplar()->CustomerDelete();
}

/*static*/
void A3CAdminToolDlg::InsertColumnHead(const char* caption_, int col_, IlvMatrix* pM_)
{
	assert(pM_);
	if(pM_)
	{
		IlvLabelMatrixItem* item = new IlvLabelMatrixItem(caption_);
		pM_->set(col_,0,item);
		pM_->setItemRelief(col_, 0);
		pM_->setItemSensitive(col_, 0, IlFalse);
		pM_->setItemGrayed(col_, 0, IlFalse);
		pM_->setItemSelected(col_, 0, IlFalse);
	}
}

void A3CAdminToolDlg::CustomerSelect()
{
	IlUShort col, row;
	if(_pCustMtx->getFirstSelected(col, row))
	{
		_pItmCustDel->setSensitive(_editMode);
	}
	else
	{
		_pItmCustDel->setSensitive(IlFalse);
	}
	_pItmCustNew->setSensitive(_editMode);
	_pCustMtx->allowEdit(_editMode);

}

void
A3CAdminToolDlg::CustomerFillMtx()
{
    int i, rows, cols=2;

	if(_pCustMtx)
	{
		const std::map<int/*CustomerID*/, boost::shared_ptr<PR::Customer> >& repo_cust = A3CProjectModule::Instance()->getCustomers();
		std::map<int/*CustomerID*/, boost::shared_ptr<PR::Customer> >::const_iterator cit;
        rows = (int)repo_cust.size();

		_pCustMtx->reinitialize(0,0);
		_pCustMtx->reinitialize(cols, rows+1);
		for(i=0;i<=rows;i++)
			_pCustMtx->resizeRow(i+1, 20);
		_pCustMtx->resizeColumn(1, 400);	// name
		_pCustMtx->resizeColumn(2, 55);		// ICAO

		InsertColumnHead("Name", 0, _pCustMtx);
		InsertColumnHead("ICAO", 1, _pCustMtx);
		
		for(i=1, cit=repo_cust.begin();cit!=repo_cust.end();i++, cit++) 
		{IlvGadgetMatrixItem* pGmi;
		 IlvTextField* pEdt;

			pEdt= new IlvTextField(TheApplication->getDisplay(),"", IlvRect(0,0,0,0));
			pEdt->setChangeCallback(OnCustomerChange);
			pEdt->setCallback(pEdt->FocusOutSymbol(), OnCustomerRename);
			pEdt->setCallback(pEdt->FocusInSymbol(), OnCustomerSelect);
			pEdt->setLabel(cit->second->_Name.c_str(), IlTrue);
			pEdt->showFrame(IlFalse);
			pGmi= new IlvGadgetMatrixItem(pEdt);
			_pCustMtx->set(0, i, pGmi);
			_pCustMtx->setItemData(0, i, (IlAny)cit->second->_CustomerID);

			pEdt= new IlvTextField(TheApplication->getDisplay(),"", IlvRect(0,0,0,0));
			pEdt->setChangeCallback(OnCustomerChange);
			pEdt->setCallback(pEdt->FocusOutSymbol(), OnCustomerRename);
			pEdt->setCallback(pEdt->FocusInSymbol(), OnCustomerSelect);
			pEdt->setAlignment(IlvCenter);
			pEdt->setLabel(cit->second->_ICAO.c_str(), IlTrue);
			pEdt->showFrame(IlFalse);
			pGmi= new IlvGadgetMatrixItem(pEdt);
			_pCustMtx->set(1, i, pGmi);
			_pCustMtx->setItemData(1, i, (IlAny)cit->second->_CustomerID);
			
		}
		_pCustMtx->reDraw();
	}
}

void A3CAdminToolDlg::CustomerNew()
{PR::Customer cust;
	
	cust._Name="<new customer>";
	cust._ICAO="<?>";
	if(A3CProjectModule::Instance()->createCustomer(cust))
	{IlvGadgetMatrixItem* pGmi;
	 IlvTextField* pEdt;
	 int i;
		
		i=_pCustMtx->rows();
		_pCustMtx->insertRow(i+1);

		pEdt= new IlvTextField(TheApplication->getDisplay(),"", IlvRect(0,0,0,0));
		pEdt->setChangeCallback(OnCustomerChange);
		pEdt->setCallback(pEdt->FocusOutSymbol(), OnCustomerRename);
		pEdt->setCallback(pEdt->FocusInSymbol(), OnCustomerSelect);
		pEdt->setLabel(cust._Name.c_str(), IlTrue);
		pEdt->showFrame(IlFalse);
		pGmi= new IlvGadgetMatrixItem(pEdt);
		_pCustMtx->set(0, i, pGmi);
		_pCustMtx->setItemData(0, i, (IlAny)cust._CustomerID);

		pEdt= new IlvTextField(TheApplication->getDisplay(),"", IlvRect(0,0,0,0));
		pEdt->setChangeCallback(OnCustomerChange);
		pEdt->setCallback(pEdt->FocusOutSymbol(), OnCustomerRename);
		pEdt->setCallback(pEdt->FocusInSymbol(), OnCustomerSelect);
		pEdt->setLabel(cust._ICAO.c_str(), IlTrue);
		pEdt->showFrame(IlFalse);
		pGmi= new IlvGadgetMatrixItem(pEdt);
		_pCustMtx->set(1, i, pGmi);
		_pCustMtx->setItemData(1, i, (IlAny)cust._CustomerID);

		_pCustMtx->reDraw();
	}
}

void A3CAdminToolDlg:: CustomerChange()
{
	
	IlUShort col;
	_pCustMtx->getFirstSelected(col, (IlUShort)_custRow);
	_custChanged=true;
	
	IlvTextField* pEdt=(IlvTextField*)(((IlvGadgetMatrixItem*)_pCustMtx->getItem(0, (IlUShort)_custRow))->getGadget());
	_custName = pEdt->getLabel();
	pEdt=(IlvTextField*)(((IlvGadgetMatrixItem*)_pCustMtx->getItem(1, (IlUShort)_custRow))->getGadget());
	_custICAO = pEdt->getLabel();

}

void A3CAdminToolDlg:: CustomerRename()
{
	if(_custChanged)
	{
		_custChanged=false;
		int custID=(int)_pCustMtx->getItemData(0, (IlUShort)_custRow);
		if(!_editMode || !A3CProjectModule::Instance()->renameCustomer(custID, _custName, _custICAO))
		{boost::shared_ptr<PR::Customer> cust;

			A3CProjectModule::Instance()->getCustomer(custID, cust);
			IlvTextField* pEdt=(IlvTextField*)(((IlvGadgetMatrixItem*)_pCustMtx->getItem(0, (IlUShort)_custRow))->getGadget());
			pEdt->setLabel(cust->_Name.c_str());
			pEdt=(IlvTextField*)(((IlvGadgetMatrixItem*)_pCustMtx->getItem(1, (IlUShort)_custRow))->getGadget());
			pEdt->setLabel(cust->_ICAO.c_str());
		}
	}
}

void A3CAdminToolDlg::CustomerDelete()
{
	IlUShort col, row;
	_pCustMtx->getFirstSelected(col, row);
	int CustId=(int)_pCustMtx->getItemData(col, row);
	if(A3CProjectModule::Instance()->deleteCustomer(CustId))
	{
		_pCustMtx->removeRow(row);
		_pCustMtx->reDraw();
		CustomerSelect();
	}
}

/*static*/
void A3CAdminToolDlg::OnAcSelect(IlvGraphic* , IlvAny )
{
	Exemplar()->AcSelect();
}
/*static*/
void A3CAdminToolDlg::OnAcNew(IlvGraphic* , IlvAny )
{
	Exemplar()->AcNew();
}
/*static*/
void A3CAdminToolDlg::OnAcDelete(IlvGraphic* , IlvAny )
{
	Exemplar()->AcDelete();
}
/*static*/
void A3CAdminToolDlg::OnAcOpenSsp(IlvGraphic* , IlvAny )
{
	Exemplar()->AcOpenSsp();
}
/*static*/
void A3CAdminToolDlg::OnAcSelectSsp(IlvGraphic* , IlvAny )
{
	Exemplar()->AcSelectSsp();
}
/*static*/
void A3CAdminToolDlg::OnAcOpenCcg(IlvGraphic* , IlvAny )
{
	Exemplar()->AcOpenCcg();
}
/*static*/
void A3CAdminToolDlg::OnAcSelectCcg(IlvGraphic* , IlvAny )
{
	Exemplar()->AcSelectCcg();
}
/*static*/
void A3CAdminToolDlg::OnAcSelectTemplate(IlvGraphic* , IlvAny )
{
	Exemplar()->AcSelectTemplate();
}

void A3CAdminToolDlg::AcFillMtx()
{int c, cols=4;

	if(_pAcMtx)
	{
		const std::map<int/*AircraftID*/, boost::shared_ptr<PR::Aircraft> >& repo_ac = A3CProjectModule::Instance()->getAircrafts();
		std::map<int/*AircraftID*/, boost::shared_ptr<PR::Aircraft> >::const_iterator cit;
		
		_pAcMtx->reinitialize(0,0);
		_pAcMtx->reinitialize(cols, 1);
		c=1;
		_pAcMtx->resizeColumn(c++, 146);	// std. spec.
		_pAcMtx->resizeColumn(c++, 146);	// ccg
		_pAcMtx->resizeColumn(c++, 143);	// template
		_pAcMtx->resizeColumn(c++, 20);	// knopp

		c=0;
		InsertColumnHead("Std. Specification", c++, _pAcMtx);
		InsertColumnHead("CCG", c++, _pAcMtx);
		InsertColumnHead("Project Template", c++, _pAcMtx);
		InsertColumnHead("", c++, _pAcMtx);
		
		for(cit=repo_ac.begin();cit!=repo_ac.end();cit++) 
		{
			AcInsertRow(cit->second.get()); 
		}
		_pAcMtx->reDraw();
	}
}

void A3CAdminToolDlg::AcInsertRow(PR::Aircraft *ac_)
{IlvGadgetMatrixItem* pGmi;
 IlvScrolledComboBox* pBox;
 IlvTextField* pEdt;
 IlvFont* pFnt;
 IlvButton* pBtn;
 boost::shared_ptr<PR::Organisation> ssp;
 boost::shared_ptr<PR::CCG> ccg;
 PR::Project tp;
 boost::shared_ptr<PR::RevIt> templ;
 int i, c;
 

	if(ac_->getAircraftID() != NULL_ID)
	{
		if(ac_->getOrgNodeID() != NULL_ID && !A3CProjectModule::Instance()->getOrgNode(ac_->getOrgNodeID(), ssp))
		{
			assert(0);
		}
		if(ssp)
		{
			assert(ssp->getLevel()==PR::SSP);
		}
		if(ac_->getCcgID() != NULL_ID && !A3CProjectModule::Instance()->getCCG(ac_->getCcgID(), ccg))
		{
			assert(0);
		}
		if(ac_->getTemplateID() != NULL_UUID)
		{
			if(A3CProjectModule::Instance()->getTemplate(ac_->getOrgNodeID(), ac_->getCcgID(), tp))
			{
				templ = tp.getRevIt(ac_->getTemplateID());
			}
			else
			{
				assert(0);
			}
		}
		
	}
	
	i=_pAcMtx->rows();
	_pAcMtx->insertRow(i+1);
	_pAcMtx->resizeRow(i+1, 20);

	pFnt=TheApplication->getDisplay()->getFont("%Arial-9-");

	c=0;
	pBox= new IlvScrolledComboBox(TheApplication->getDisplay(), IlvRect(0,0,0,0), "");
	pBox->setCallback(pBox->FocusInSymbol(), OnAcSelect);
	pBox->setCallback(pBox->OpenListSymbol(), OnAcOpenSsp);
	pBox->setCallback(OnAcSelectSsp);
	if(pFnt)
	{
		pBox->setFont(pFnt);
	}
	pBox->setLabel(!ssp ? "<select>" : ssp->getName().c_str(), IlTrue);
	pBox->showFrame(IlFalse);
	pBox->enableLargeList();
	pBox->setEditable(IlFalse);
	pGmi= new IlvGadgetMatrixItem(pBox);
	_pAcMtx->set(c, i, pGmi);
	_pAcMtx->setItemData(c, i, (IlAny)ac_->getAircraftID());

	c++;
	pBox= new IlvScrolledComboBox(TheApplication->getDisplay(), IlvRect(0,0,0,0), "");
	pBox->setCallback(pBox->FocusInSymbol(), OnAcSelect);
	pBox->setCallback(pBox->OpenListSymbol(), OnAcOpenCcg);
	pBox->setCallback(OnAcSelectCcg);
	if(pFnt)
	{
		pBox->setFont(pFnt);
	}
	pBox->setLabel(!ccg ? "<select>" : ccg->getName().c_str(), IlTrue);
	pBox->showFrame(IlFalse);
	pBox->enableLargeList();
	pBox->setEditable(IlFalse);
	pGmi= new IlvGadgetMatrixItem(pBox);
	_pAcMtx->set(c, i, pGmi);
	_pAcMtx->setItemData(c, i, (IlAny)ac_->getAircraftID());

	c++;
	pEdt= new IlvTextField(TheApplication->getDisplay(),"", IlvRect(0,0,0,0));
	pEdt->setCallback(pEdt->FocusInSymbol(), OnAcSelect);
	if(ac_->getTemplateID()!=NULL_UUID)
	{
		std::stringstream lbl;
		lbl << tp.getName() << " " << templ->getRevision() << "/" << templ->getIteration();
		pEdt->setLabel(lbl.str().c_str(), IlTrue);
	}
	else
	{
		pEdt->setLabel("<not set>", IlTrue);
	}
	pEdt->showFrame(IlFalse);
	pEdt->setFocusable(IlFalse);
	pGmi= new IlvGadgetMatrixItem(pEdt);
	_pAcMtx->set(c, i, pGmi);
	_pAcMtx->setItemData(c, i, (IlAny)ac_->getAircraftID());

	c++;
	pBtn = new IlvButton(TheApplication->getDisplay(),"...", IlvRect(0,0,0,0));
	if(pFnt)
	{
		pBtn->setFont(pFnt);
	}
	pBtn->setCallback(pEdt->FocusInSymbol(), OnAcSelect);
	pBtn->setCallback(OnAcSelectTemplate);
	pBtn->setClientData( (IlAny)(ac_->getOrgNodeID()!=NULL_ID && ac_->getCcgID()!=NULL_ID ? 1 : 0));
	pGmi= new IlvGadgetMatrixItem(pBtn);
	_pAcMtx->set(c, i, pGmi);
	_pAcMtx->setItemData(c, i, (IlAny)ac_->getAircraftID());
	
}

void A3CAdminToolDlg::AcSelect()
{
	IlUShort col, row;
	if(_pAcMtx->getFirstSelected(col, row))
	{
		_pItmAcDel->setSensitive(_editMode);
	}
	else
	{
		_pItmAcDel->setSensitive(IlFalse);
	}
	_pItmAcNew->setSensitive(_editMode);
	
	_pAcMtx->allowEdit(_editMode);
	IlUShort i, rows=_pAcMtx->rows();
	IlvScrolledComboBox* pBox;
	IlvButton* pBtn;
	for(i=1; i<rows; i++)
	{
		pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(0, i))->getGadget());
		pBox->setActive(_editMode);
		pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(1, i))->getGadget());
		pBox->setActive(_editMode);
		pBtn = (IlvButton*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(3, i))->getGadget());
		pBtn->setActive( pBtn->getClientData() && _editMode);
	}

}

void A3CAdminToolDlg::AcNew()
{PR::Aircraft ac;
	
	if(A3CProjectModule::Instance()->createAC(ac))
	{
		AcInsertRow(&ac);
		_pAcMtx->reDraw();
	}
}

void A3CAdminToolDlg::AcDelete()
{
	IlUShort col, row;
	_pAcMtx->getFirstSelected(col, row);
	int AcId=(int)_pAcMtx->getItemData(col, row);
	if(A3CProjectModule::Instance()->deleteAC(AcId))
	{
		_pAcMtx->removeRow(row);
		_pAcMtx->reDraw();
		AcSelect();
	}
}

void A3CAdminToolDlg::AcOpenSsp()
{
	IlUShort col, row;
	std::list<const boost::shared_ptr<PR::Organisation> > lstorg;
	std::list<const boost::shared_ptr<PR::Organisation> >::const_iterator it;
	_pAcMtx->getFirstSelected(col, row);
	IlvScrolledComboBox* pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(col, row))->getGadget());
	pBox->empty();
	A3CProjectModule::Instance()->getOrg(PR::SSP, lstorg);
	for(it=lstorg.begin(); it!=lstorg.end(); it++)
	{
		pBox->addLabel((*it)->getName().c_str(), (IlAny)(*it)->getOrgNodeID());
	}
}

void A3CAdminToolDlg::AcOpenCcg()
{
	IlUShort col, row;
	_pAcMtx->getFirstSelected(col, row);
	IlvScrolledComboBox* pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(col, row))->getGadget());
	pBox->empty();
	const std::map<int/*CcgID*/, boost::shared_ptr<PR::CCG> >& ccgs = A3CProjectModule::Instance()->getCCGs();
	std::map<int/*CcgID*/, boost::shared_ptr<PR::CCG> >::const_iterator it;
	for(it=ccgs.begin(); it!=ccgs.end(); it++)
	{
		pBox->addLabel(it->second->getName().c_str(), (IlAny)it->second->getCcgID());
	}

}

void A3CAdminToolDlg::AcSelectSsp()
{
	IlUShort col, row;
	_pAcMtx->getFirstSelected(col, row);
	IlvScrolledComboBox* pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(col, row))->getGadget());
	IlShort ndx=pBox->getFirstSelectedItem();
	int orgid = (int)pBox->getItemData(ndx);
	int acid = (int)_pAcMtx->getItemData(col, row);

	boost::shared_ptr<PR::Aircraft> ac;
	if(A3CProjectModule::Instance()->getAircraft(acid, ac))
	{
		int old=ac->getOrgNodeID();
		ac->setOrgNodeID(orgid);
		if(!A3CProjectModule::Instance()->updateAC(ac))
		{boost::shared_ptr<PR::Organisation> ssp;

			ac->setOrgNodeID(old);
			A3CProjectModule::Instance()->getOrgNode(ac->getOrgNodeID(), ssp);
			pBox->setLabel(ssp ? ssp->getName().c_str() : "<select>" );
		}
		else
		{
			IlvButton* pBtn = (IlvButton*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(3, row))->getGadget());
			pBtn->setClientData( (IlAny)(ac->getOrgNodeID()!=NULL_ID && ac->getCcgID()!=NULL_ID ? 1 : 0));
			pBtn->setActive(ac->getOrgNodeID()!=NULL_ID && ac->getCcgID()!=NULL_ID && _editMode);
		}
	}
	else
		assert(0);
}
 
void A3CAdminToolDlg::AcSelectCcg()
{
	IlUShort col, row;
	_pAcMtx->getFirstSelected(col, row);
	IlvScrolledComboBox* pBox = (IlvScrolledComboBox*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(col, row))->getGadget());
	IlShort ndx=pBox->getFirstSelectedItem();
	int ccgid = (int)pBox->getItemData(ndx);
	int acid = (int)_pAcMtx->getItemData(col, row);

	boost::shared_ptr<PR::Aircraft> ac;
	if(A3CProjectModule::Instance()->getAircraft(acid, ac))
	{
		int old=ac->getCcgID();
		ac->setCcgID(ccgid);
		if(!A3CProjectModule::Instance()->updateAC(ac))
		{boost::shared_ptr<PR::CCG> ccg;

			ac->setCcgID(old);
			A3CProjectModule::Instance()->getCCG(ac->getCcgID(), ccg);
			pBox->setLabel(ccg ? ccg->getName().c_str() : "<select>" );
		}
		else
		{
			IlvButton* pBtn = (IlvButton*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(3, row))->getGadget());
			pBtn->setClientData( (IlAny)(ac->getOrgNodeID()!=NULL_ID && ac->getCcgID()!=NULL_ID ? 1 : 0));
			pBtn->setActive(ac->getOrgNodeID()!=NULL_ID && ac->getCcgID()!=NULL_ID && _editMode);
		}
	}
	else
		assert(0);
}

void A3CAdminToolDlg::AcSelectTemplate()
{std::string tmpid;
 bool valid;
 std::string tmpname;

	IlUShort col, row;
	_pAcMtx->getFirstSelected(col, row);
	int acid = (int)_pAcMtx->getItemData(col, row);

	boost::shared_ptr<PR::Aircraft> ac;
	if(A3CProjectModule::Instance()->getAircraft(acid, ac))
	{std::list<boost::shared_ptr<Pace::A3CStorage::PR::Project> > lstproj;

		if(A3CProjectModule::Instance()->getProjects(ac->getOrgNodeID(), ac->getCcgID(), lstproj, true))
		{
			ATSelectTemplateDlg(valid, tmpname, tmpid, lstproj);
			if(valid)
			{
				IlvTextField* pEdt = (IlvTextField*)(((IlvGadgetMatrixItem*)_pAcMtx->getItem(2, row))->getGadget());
				pEdt->setLabel(tmpname.c_str(), true);
				ac->setTemplateID(tmpid);
				A3CProjectModule::Instance()->updateAC(ac);
			}
		}
		else
		{
			assert(0);
		}
		A3CProjectModule::Instance()->clearList(lstproj);
	}
	else
		assert(0);
	
	
}
