//                                              ______
//                                                   `
// (c) 2004 by                              P A C E   /\ 
//                                                      
// Aerospace Engineering and Information Technology GmbH
//                                                      
// ---------------------------------------------------------------------------
// $Workfile: AutoCADExporter.cxx $
// ---------------------------------------------------------------------------
//                                                
// A3C class for 2D graphics export to AutoCAD.
// (Origin: Jul 2004 Christian R. Krug)
//
// $Author: Juergenf $
// $Date: 31.03.05 18:43 $
// $Revision: 7 $
// $Modtime: 31.03.05 11:22 $
//

//#include "../../include/CabinPCH.hxx"
//#pragma hdrstop

#include "../../include/AutoCADExport/AutoCADExporter.hxx"

#include "../../include/AutoCADExport/ExportInterface.hxx"

#include <Modules/Modeler/Drafting/DraftCopyController.hxx>
#include <Modules\Modeler\Drafting\DraftBuilder.hxx>
#include <Modules\Modeler\Drafting\Draft.hxx>
#include <Modules\Modeler\Drafting\DraftCopyFilter.hxx>
#include <Modules\Modeler\Drafting\ExportCopyFilter.hxx>
//#include <Modules/OpenDWGFilter/DWGExport.hxx>
//#include <Modules/OpenDWGFilter/DWGImport.hxx>
#include <Modules/DWGFilter/DWGImport.hxx>
#include <Modules/DWGFilter/DWGExport.hxx>

using namespace Pace;
using namespace cabinlayout;

AutoCADExporter::AutoCADExporter()
{
	_InitFormatList();
}

bool AutoCADExporter::export(IlvView * pView_,
							 const PaceString & rstrFileName_,
							 const PaceString & rstrFileExtension_,
							 PaceDrawingPort const *  pDrawPort_,
							 const IlvTransformer * const)
{static int i=1;
	
	if(i==1) //filtered
	{
		Modeler::DraftBuilder builder;
		Modeler::DraftCopyController copier(&builder);
		ExportInterface internExporter(&copier);
		if (internExporter.ExportAll())
		{	
			std::auto_ptr<Modeler::Draft> ptrDraft = builder.GetResult();
			DWGFilter::DWGExport dwgExporter;
			Modeler::DraftCopyController writer(&dwgExporter);

			Modeler::ExportCopyFilter exportFilter;
			exportFilter.SetShortBlockNames(8);
			exportFilter.SetShortLayerNames(8);
			writer.SetCopyFilter(&exportFilter);

			Modeler::DraftCopyFilter draftFilter;
			draftFilter.SetAllowPaperspace(false);
			draftFilter.SetFilterFrozenLayer(true);
			draftFilter.SetUseFilterRules(true);
			draftFilter.UseConstBlockRefMarker("BlockRef--");

			if(1)
				ptrDraft->CopyFiltered(&writer, &draftFilter);
			else
				ptrDraft->CopyAll(&writer);

			dwgExporter.SaveToFile(rstrFileName_);

			return true;
		}
		else
			return false;
	}

	if(i==2) // unfiltered, creating paperspaces
	{
		// init DWG export
		DWGFilter::DWGExport dwgExporter;
		Modeler::DraftCopyController writer(&dwgExporter);
		
		// add cabin layout elements
		ExportInterface internExporter(&writer);
		if (internExporter.ExportAll())
		{	
			// write export file
			dwgExporter.SaveToFile(rstrFileName_);

			return true;
		}
		else
			return false;
	}
	
	
	
	return false;
}
							
void AutoCADExporter::_InitFormatList()
{
	// initialize _exportFormatList
	
	ExportFormatInfo efi1;
	efi1.strName = "DWG";
	efi1.strDescription = "&FileFormatDescriptionDWG";
	efi1.strFileExtensions.push_back("dwg");
	efi1.graphicsType = PA_VECTOR_GRAPHICS;
	efi1.bCanResize = false;
	efi1.pExporterInstance = this;
	_exportFormatList.push_back(efi1);

	ExportFormatInfo efi2;
	efi2.strName = "DXF";
	efi2.strDescription = "&FileFormatDescriptionDXF";
	efi2.strFileExtensions.push_back("dxf");
	efi2.graphicsType = PA_VECTOR_GRAPHICS;
	efi2.bCanResize = false;
	efi2.pExporterInstance = this;
	_exportFormatList.push_back(efi2);
}
