use Term::ReadKey;
use Env qw($SSDIR);;
use Cwd;
use File::Find;
do 'getSrcConfig.pl';

$vaultExec=$ENV{SGVAULT};
if($vaultExec eq '') {die "Error: Set environment variable SGVAULT\n";} 
$vaultExec.='\\vault';

$ssExec=$ENV{SSBIN};
if($ssExec eq '') {die "Error: Set environment variable SSBIN\n";} 
$ssExec.='\\ss';


my %fileProperties;

sub getFileProperties {
	my @stats = stat( $File::Find::name);
	$fileProperties{$File::Find::name}=[$stats[7],$stats[9]]; # extract file size and file modification date
}

sub compareFileProperties {
	if(-d $File::Find::name ||
	   $File::Find::name =~ /_sgvault/ || 
	   $File::Find::name =~ /vssver\.scc/) {
		return;
	}
	if(!defined($fileProperties{$File::Find::name})) {
		print "new: $File::Find::name\n";	
	} else {
		my @stats = stat($File::Find::name);
		if($fileProperties{$File::Find::name}[0] != $stats[7] ||
		   $fileProperties{$File::Find::name}[1] != $stats[9]) {
			print "chg: $File::Find::name\n";	
		}	
	}	
}

sub isLoggedIn {
	open(FILE,'-|',"\"$vaultExec\" listrepositories") || die "Cannot exec $vaultExec\n";
	my $str;
	while(my $s=<FILE>) {$str .= $s;}
	return (($str =~ /result success=.yes./) ? 1 : 0);
}

sub signonVault {
	my $user=$ENV{USERNAME};
	print "Enter password for user '$user': ";
	ReadMode 'noecho';
	my $passwd = ReadLine 0;
	chomp $passwd;
	ReadMode 'normal';	
	print "\n";
	open(FILE,'-|',"\"$vaultExec\" rememberlogin -user $user -password \"$passwd\" -host $VaultHost -repository $VaultRepository") || die "Cannot exec $vaultExec\n";
	my $str;
	while(my $s=<FILE>) {$str.=$s;}
	close(FILE);
	die "Cannot log on vault\n$str\n" unless ($str =~ /result success=.yes./);
}

sub signoffVault {
	open(FILE,'-|',"\"$vaultExec\" forgetlogin")|| die "Cannot exec $vaultExec\n";
	my $str;
	while(my $s=<FILE>) {$str.=$s;}
	close(FILE);
	die "Cannot log off vault\n$str\n" unless ($str =~ /result success=.yes./);
}

sub getFromVault {
	my ($folder,$workingdir,$overwrite,$version)=@_;
	my $str;
	print "Getting '$folder'" . ($version eq "" ? "" : " (Version $version)")." to '$workingdir'...";
	open(FILE,'-|',"\"$vaultExec\" setworkingfolder \"$folder\" \"$workingdir\"") || die "Cannot exec $vaultExec\n";
	while(my $s=<FILE>) {$str.=$s;}
	close(FILE);
	die "Cannot unsetworkingfolder $folder\n$str\n" unless ($str =~ /result success=.yes./);
	
	$str='';
	if($version eq '') {
		open(FILE,'-|',"\"$vaultExec\" get -makewritable -setfiletime modification -merge " .($overwrite ? 'overwrite' : 'automatic'). " \"$folder\"") || die "Cannot exec $vaultExec\n";
		while(my $s=<FILE>) {$str.=$s;}
		close(FILE);
		die "Cannot get $folder\n$str\n" unless ($str =~ /result success=.yes./);
	} else {
		open(FILE,'-|',"\"$vaultExec\" getlabel -setfiletime modification -destpath  \"$workingdir\" -makewritable -merge overwrite \"$folder\" \"$version\" ") || die "Cannot exec $vaultExec\n";
		while(my $s=<FILE>) {$str.=$s;}
		close(FILE);
		die "Cannot get $folder\n$str\n" unless ($str =~ /result success=.yes./);
	}
	print "ok\n";
}

sub getFromSS {
	my ($folder,$workingdir,$version)=@_;
	my $str;
	print "Getting '$folder'" . ($version eq '' ? '' : " (Version $version)")." to '$workingdir'...";
	open(FILE,'-|',"\"$ssExec\" get \"$folder\" -R -GWRTM -I-Y -GL\"$workingdir\"" . ($version eq "" ? "" : " -VL\"$version\""));
	while(my $s=<FILE>) {$str.=$s;}
	close(FILE);
	print "ok\n";
}	 

my $get_newest = 0;
my $overwrite = 1;
foreach $param (@ARGV) {
	if($param eq 'newest') {$get_newest = 1;}
	if($param eq 'merge') {$overwrite = 0;}
	if($param eq 'signoff') {signoffVault();exit(0);}
}

if($overwrite == 0 && $get_newest == 0) {
	print "Warning: -merge- implies -newest- !!!\n";
	$get_newest = 1;	
}

if(!isLoggedIn()) {
	signonVault();
} else {
	print "You are already logged in as user $ENV{USERNAME}\n";	
}

find(\&getFileProperties, getcwd);

for my $i (0 .. scalar(@GetFromSS)-1) {
	if(!$get_newest || $GetFromSS[$i][1] eq '') {	
		getFromSS($GetFromSS[$i][0],getcwd,$GetFromSS[$i][1]);
	}
}

for my $i (0 .. scalar(@GetFromVault)-1) {
	if(!$get_newest || $GetFromVault[$i][2] eq '') {
		getFromVault($GetFromVault[$i][0],getcwd . (defined($GetFromVault[$i][1]) && $GetFromVault[$i][1] ne '' ? '/' . $GetFromVault[$i][1]: ''),$overwrite,$GetFromVault[$i][2]);
	}
}
print "\nDifferences to previous content:\n";
find(\&compareFileProperties, getcwd);

my $workingdir=getcwd;
system("attrib -r $workingdir\\bin\\release\\*.*");
system("attrib -r $workingdir\\bin\\debug\\*.*");

